/* $Id: core.v 58 2010-08-01 19:15:10Z matutani $ */
`include "../design/define.h"
module core (
	ordy,
	inject,
	dest,
	vch,

	odata,
	ovalid,
	ovch,
	irdy,

	clear,
	count,
	idata,
	ivalid,

	slot,

	clk,
	rst_
);
parameter	COREID = 0;

output	[`VCH:0]	ordy;	/* Ready for packet injection */
input			inject;	/* Start packet injection */
input 	[`NODEW:0]	dest;	/* Destination address */
input	[`VCHW:0]	vch;	/* Virtual channel (VC) ID */

output 	[`DATAW:0]	odata;	/* Output data to router */
output			ovalid;	/* Valid signal to router */
output	[`VCHW:0]	ovch;	/* Requested VC ID */
input	[`VCH:0]	irdy;	/* Ready for packet injection to router */

input			clear;	/* Reset the packet counter */
output 	[`PKTCNTW:0]	count;	/* Packet counter value */
input 	[`DATAW:0]	idata;	/* Input data from router */
input			ivalid;	/* Valid signal from router */

input			slot;	/* Pulse generated when my time-slot starts */

input			clk;
input			rst_;

reg	[`PKTCNTW:0]	count;	/* Packet counter */
reg	[`VCH:0]	ordy;

reg 	[`NODEW:0]	odest;	/* Store the destination address */
reg 	[`VCHW:0]	ovch;	/* Store the VC ID */
reg	[1:0]		state;	/* States of packet generator */
reg	[`PKTLENW:0]	len;	/* Packet length counter */
reg 	[`DATAW:0]	odata;
reg			ovalid;
reg			pending;/* Pending injection request */

reg	[23:0]		rnd24;
wire	[23:0]		lfsr24;

/*
 * LFSR for 24-bit
 * Four taps : 80000D (1000_0000_0000_0000_0000_1101)
 * http://www.madlabo.com/mad/edat/mathematic/M/index.htm#SEC5
 */
assign	lfsr24	= {rnd24[22:0], rnd24[23] ^ rnd24[3] ^ rnd24[2] ^ rnd24[0]};

/*
 * States of packet generator
 */
`define PKTGEN_HEAD	2'b00
`define PKTGEN_DATA	2'b01
`define PKTGEN_TAIL	2'b10

/*
 * Packet counter
 */
always @(posedge clk or negedge rst_) begin
	if (!rst_) begin
		count	<= 0;
	end else begin
		if (clear) begin
			count	<= 0;		/* Clear the counter */
		end else if (ivalid && |idata) begin
			count	<= count + 1;	/* Increment the counter */
		end
	end
end

/*
 * Packet generator
 */
always @(posedge clk or negedge rst_) begin
	if (!rst_) begin
		ordy	<= 0;
		state	<= `PKTGEN_HEAD;
		len	<= 0;
		odata	<= 0;
		ovalid	<= `Disable;
		odest	<= 0;
		ovch	<= 0;
		rnd24	<= 24'h5a5a5a; 	/* Random seed */
		pending	<= `Disable;
	end else begin
		case (state)
		/*
		 * Send head flit
		 */
		`PKTGEN_HEAD: begin
			if ((inject || pending) && irdy[vch] && slot) begin
				odata	<= {`TYPE_HEAD, rnd24, len, dest};
				ovalid	<= `Enable;
				len	<= 1;
				state	<= `PKTGEN_DATA;
				ordy[vch]<= `Disable;
				odest	<= dest;
				ovch	<= vch;
				rnd24	<= lfsr24;	/* Next random */
				if (!inject && pending)
					pending	<= `Disable;
			end else if (inject && irdy[vch] && !slot) begin
				/* It is not my time-slot edge unfortunately */
				pending	<= `Enable;
			end else begin
				if (inject)
					pending	<= `Enable;
				if (ovalid) begin
					odata	<= 0;
					ovalid	<= `Disable;
				end
				if (pending)
					ordy	<= `VCH_P1'b0;
				else if (ordy != irdy)
					ordy	<= irdy;
			end
		end
		/*
		 * Send body flit(s)
		 */
		`PKTGEN_DATA: begin
			if (irdy[ovch]) begin
				odata	<= {`TYPE_DATA, rnd24, len, odest};
				ovalid	<= `Enable;
				rnd24	<= lfsr24;	/* Next random */
				len	<= len + 1;
				if (len + 1 == `PKTLEN) begin
					state	<= `PKTGEN_TAIL;
				end 
			end else if (ovalid) begin
				odata	<= 0;
				ovalid	<= `Disable;
			end
		end
		/*
		 * Send tail flit
		 */
		`PKTGEN_TAIL: begin
			if (irdy[ovch]) begin
				odata	<= {`TYPE_TAIL, rnd24, len, odest};
				ovalid	<= `Enable;
				len	<= 0;
				state	<= `PKTGEN_HEAD;
				rnd24	<= lfsr24;	/* Next random */
			end else if (ovalid) begin
				odata	<= 0;
				ovalid	<= `Disable;
			end
		end
		endcase
	end
end

endmodule
