/* $Id: cube.v 121 2010-08-15 06:58:37Z matutani $ */
`include "../design/define.h"
module cube (
	idata,
	odata,
	sel,
	wr,

	probe,

	bidata_0,
	bsel_0,
	bodata_1,
	bsel_1,

	bodata_2,
	bsel_2,
	bidata_3,
	bsel_3,

	bodata_4,
	bsel_4,
	bidata_5,
	bsel_5,

	bidata_6,
	bsel_6,
	bodata_7,
	bsel_7,

	sidata_0,
	sodata_0,
	ssel_0,

	sidata_1,
	sodata_1,
	ssel_1,

	sidata_2,
	sodata_2,
	ssel_2,

	sidata_3,
	sodata_3,
	ssel_3,

	clk,
	rst_
);
parameter		CHIPID = 0;
`ifdef __SIM__
integer			count;
`endif /* __SIM__ */

input	[3:0]		idata;
output	[2:0]		odata;
input	[1:0]		sel;
input			wr;

output	[8:0]		probe;

/* From upper to here (Bubble, downstream) */ 
input	[`TRDATAW:0]	bidata_0;
output	[`TRSELW:0]	bsel_0;
output	[`VCH:0]	bodata_1;
output	[`TRSELW:0]	bsel_1;

/* From here to lower (Bubble, downstream) */ 
output	[`TRDATAW:0]	bodata_2;
output	[`TRSELW:0]	bsel_2;
input	[`VCH:0]	bidata_3;
output	[`TRSELW:0]	bsel_3;

/* From here to upper (Bubble, upstream) */ 
output	[`TRDATAW:0]	bodata_4;
output	[`TRSELW:0]	bsel_4;
input	[`VCH:0]	bidata_5;
output	[`TRSELW:0]	bsel_5;

/* From lower to here (Bubble, upstream) */ 
input	[`TRDATAW:0]	bidata_6;
output	[`TRSELW:0]	bsel_6;
output	[`VCH:0]	bodata_7;
output	[`TRSELW:0]	bsel_7;

/* TX/RX 0 (Sbus) */
input	[`TRDATAW:0]	sidata_0;
output	[`TRDATAW:0]	sodata_0;
output	[`TRSELW:0]	ssel_0;

/* TX/RX 1 (Sbus) */
input	[`TRDATAW:0]	sidata_1;
output	[`TRDATAW:0]	sodata_1;
output	[`TRSELW:0]	ssel_1;

/* TX/RX 2 (Sbus) */
input	[`TRDATAW:0]	sidata_2;
output	[`TRDATAW:0]	sodata_2;
output	[`TRSELW:0]	ssel_2;

/* TX/RX 3 (Sbus) */
input	[`TRDATAW:0]	sidata_3;
output	[`TRDATAW:0]	sodata_3;
output	[`TRSELW:0]	ssel_3;

input			clk;
input			rst_;		/* Active-low */

/*
 * Control registers
 */
wire	[2:0]		odata_0;	/* odata for Reg 0 */
wire	[2:0]		odata_1;	/* odata for Reg 1 */
wire	[2:0]		odata_2;	/* odata for Reg 2 */
wire	[2:0]		odata_3;	/* odata for Reg 3 */
/* Reg 0 */
reg			csel;		/* Target core (left or right) */
reg			inject_0;	/* Inject a packet from Core 0 */
reg			inject_1;	/* Inject a packet from Core 1 */
reg			clear_0;	/* Clear packet counter of Core 0 */
reg			clear_1;	/* Clear packet counter of Core 1 */
wire	[`VCH:0]	ordy_0;		/* Injection ready from Core 0 */
wire	[`VCH:0]	ordy_1;		/* Injection ready from Core 1 */
reg                     dummy;		/* Dummy mode for avoding wire removal*/
/* Reg 1 */
reg 	[`NODEW:0]	dest_0;		/* Dest addr of Core 0 packets */
reg 	[`NODEW:0]	dest_1;		/* Dest addr of Core 1 packets */
wire 	[`PKTCNTW:0]	count_0;	/* Packet counter value of Core 0 */
wire 	[`PKTCNTW:0]	count_1;	/* Packet counter value of Core 1 */
/* Reg 2 */
reg	[`VCHW:0]	vch_0;		/* VC ID of Core 0 packets */
reg	[`VCHW:0]	vch_1;		/* VC ID of Core 1 packets */
reg			burst_0;	/* Burst mode switch of Core 0 */ 
reg			burst_1;	/* Burst mode switch of Core 1 */ 
reg                     mode_0;         /* Point-to-point or sbus */
reg	[2:0]		count3;		/* Counter value (3-bit of 45-bit) */
reg	[3:0]		ptr;		/* Counter pointer (0 ... 14) */
/* Reg 3 */
reg	[`NODEW:0]	id_0;		/* Router ID of Core 0 */
reg	[`NODEW:0]	id_1;		/* Router ID of Core 1 */
reg			top;		/* This chip is top of package */
reg			bottom;		/* This chip is bottom of package */

/* Time Slot Control */
wire			ts_control;
wire			ts_start;	/* Pulse generated when my TS starts */
reg			ts_control_last;/* The previous value of ts_control */

/*
 * Wire (Router <--> P2P/Sbus switch)
 */
wire	[`DATAW:0]	r0_idata_0;
wire			r0_ivalid_0;
wire	[`VCHW:0]	r0_ivch_0;

wire	[`DATAW:0]	r0_odata_0;
wire			r0_ovalid_0;
wire	[`VCHW:0]	r0_ovch_0;
wire	[`VCH:0]	r0_ordy_0;

wire	[`DATAW:0]	r1_idata_0;
wire			r1_ivalid_0;
wire	[`VCHW:0]	r1_ivch_0;

wire	[`DATAW:0]	r1_odata_0;
wire			r1_ovalid_0;
wire	[`VCHW:0]	r1_ovch_0;
wire	[`VCH:0]	r1_ordy_0;

/*
 * Wire (Core <--> P2P/Sbus switch)
 */
wire	[`DATAW:0]	c0_idata;
wire			c0_ivalid;
wire	[`VCHW:0]	c0_ivch;
wire	[`VCH:0]	c0_irdy;

wire	[`DATAW:0]	c0_odata;
wire			c0_ovalid;
wire	[`VCHW:0]	c0_ovch;

wire	[`DATAW:0]	c1_idata;
wire			c1_ivalid;
wire	[`VCHW:0]	c1_ivch;
wire	[`VCH:0]	c1_irdy;

wire	[`DATAW:0]	c1_odata;
wire			c1_ovalid;
wire	[`VCHW:0]	c1_ovch;

/*
 * Wire (Router <--> Router)
 */
wire	[`DATAW:0]	r0_odata_1;
wire			r0_ovalid_1;
wire	[`VCHW:0]	r0_ovch_1;
wire	[`VCH:0]	r1_oack_1;

wire	[`DATAW:0]	r1_odata_1;
wire			r1_ovalid_1;
wire	[`VCHW:0]	r1_ovch_1;
wire	[`VCH:0]	r0_oack_1;

/*
 * Wire (Router <--> TX/RX)
 */
wire	[`DATAW:0]	r0_idata_2;
wire			r0_ivalid_2;
wire	[`VCHW:0]	r0_ivch_2;
wire	[`VCH:0]	r0_oack_2;

wire	[`DATAW:0]	r0_odata_2;
wire			r0_ovalid_2;
wire	[`VCHW:0]	r0_ovch_2;
wire	[`VCH:0]	r0_iack_2;

wire	[`DATAW:0]	r1_idata_2;
wire			r1_ivalid_2;
wire	[`VCHW:0]	r1_ivch_2;
wire	[`VCH:0]	r1_oack_2;

wire	[`DATAW:0]	r1_odata_2;
wire			r1_ovalid_2;
wire	[`VCHW:0]	r1_ovch_2;
wire	[`VCH:0]	r1_iack_2;

/*
 * Wire (Sbus <--> P2P/Sbus switch)
 */
wire    [`DATAW:0]      sc_idata;
wire                    sc_ivalid;
wire    [`VCH:0]        sc_irdy;

wire    [`DATAW:0]      sc_odata;
wire                    sc_ovalid;
wire    [`VCH:0]        sc_ordy;

/*
 * Wire (Sbus <--> TX/RX)
 */
wire	[`TRDATAW:0]	str_odata;
wire	[`TRSELW:0]	str_sel;

/*
 * For debugging
 */
`ifdef __SIM__
initial begin
        count   <= 0;
end
always @(posedge clk) begin
        count   <= count + 1;
end
`endif /* __SIM__ */

/*
 * Control register output
 */
assign	odata	= (sel == 0) ? odata_0 : 
		  (sel == 1) ? odata_1 :
		  (sel == 2) ? odata_2 : 
		  (sel == 3) ? odata_3 : 3'b0;

/*
 * Control register 0
 */
always @(posedge clk or negedge rst_) begin
	if (!rst_) begin
		csel	<= 0;
		inject_0<= `Disable;
		inject_1<= `Disable;
		clear_0	<= `Disable;
		clear_1	<= `Disable;
		dummy	<= `Disable;
	end else if (sel == 0 && wr) begin
		csel	<= idata[0];		/* Select a target core */
		dummy	<= idata[2];		/* Dummy mode */
		if (csel == 0) begin
			if (burst_0)		/* Inject packet from Core 0 */
				inject_0<= `Enable;
			else
				inject_0<= idata[1];
			clear_0	<= idata[3];	/* Clear counter of Core 0 */
			clear_1	<= `Disable;
		end
		if (csel == 1) begin
			if (burst_1)		/* Inject packet from Core 1 */
				inject_1<= `Enable;
			else
				inject_1<= idata[1];
			clear_0	<= `Disable;
			clear_1	<= idata[3];	/* Clear counter of Core 1 */
		end
	end else begin
		if (inject_0 && c0_irdy[vch_0] && !burst_0)
			inject_0<= `Disable;
		if (inject_1 && c1_irdy[vch_1] && !burst_1)
			inject_1<= `Disable;
		if (clear_0)
			clear_0	<= `Disable;
		if (clear_1)
			clear_1	<= `Disable;
	end
end
assign	odata_0[0]	= (csel == 0) ? ordy_0[0] :
			  (csel == 1) ? ordy_1[0] : `Disable;
assign	odata_0[1]	= (csel == 0) ? ordy_0[1] :
			  (csel == 1) ? ordy_1[1] : `Disable;
assign	odata_0[2]	= csel;

/*
 * Control register 1
 */
always @(posedge clk or negedge rst_) begin
	if (!rst_) begin
		dest_0	<= 0;
		dest_1	<= 0;
	end else if (sel == 1 && wr) begin
		if (csel == 0)
			dest_0	<= idata[3:0];	/* Dest addr of Core 0 packet */
		if (csel == 1)
			dest_1	<= idata[3:0];	/* Dest addr of Core 1 packet */
	end
end
assign	odata_1[2:0]	= (csel == 0) ? count_0[2:0] :
			  (csel == 1) ? count_1[2:0] : 0;

/*
 * Control register 2
 */
always @(posedge clk or negedge rst_) begin
	if (!rst_) begin
		vch_0	<= 0;
		vch_1	<= 0;
		burst_0	<= `Disable;
		burst_1	<= `Disable;
		mode_0	<= `Disable;
		count3	<= 0;
		ptr	<= 0;
	end else if (sel == 2 && wr) begin
		if (csel == 0) begin
			vch_0	<= idata[0];	/* VC ID of Core 0 packet */
			burst_0	<= idata[1];	/* Burst mode of Core 0 */
			//burst_1	<= `Disable;
              	        mode_0  <= idata[2];    /* Point-to-point or sbus */
			if (idata[3]) begin	/* Count value read of Core 0 */
				case (ptr)
				4'd0:	count3	<= count_0[2:0];
				4'd1:	count3	<= count_0[5:3];
				4'd2:	count3	<= count_0[8:6];
				4'd3:	count3	<= count_0[11:9];
				4'd4:	count3	<= count_0[14:12];
				4'd5:	count3	<= count_0[17:15];
				4'd6:	count3	<= count_0[20:18];
				4'd7:	count3	<= count_0[23:21];
				4'd8:	count3	<= count_0[26:24];
				4'd9:	count3	<= count_0[29:27];
				4'd10:	count3	<= count_0[32:30];
				4'd11:	count3	<= count_0[35:33];
				4'd12:	count3	<= count_0[38:36];
				4'd13:	count3	<= count_0[41:39];
				4'd14:	count3	<= count_0[44:42];
				default:count3	<= 0;
				endcase
				if (ptr >= 14)	ptr	<= 0;
				else		ptr	<= ptr + 1;
			end else begin
				count3	<= 0;
				ptr	<= 0;
			end
		end
		if (csel == 1) begin
			vch_1	<= idata[0];	/* VC ID of Core 1 packet */
			//burst_0	<= `Disable;
			burst_1	<= idata[1];	/* Burst mode of Core 1 */
			if (idata[3]) begin	/* Count value read of Core 1 */
				case (ptr)
				4'd0:	count3	<= count_1[2:0];
				4'd1:	count3	<= count_1[5:3];
				4'd2:	count3	<= count_1[8:6];
				4'd3:	count3	<= count_1[11:9];
				4'd4:	count3	<= count_1[14:12];
				4'd5:	count3	<= count_1[17:15];
				4'd6:	count3	<= count_1[20:18];
				4'd7:	count3	<= count_1[23:21];
				4'd8:	count3	<= count_1[26:24];
				4'd9:	count3	<= count_1[29:27];
				4'd10:	count3	<= count_1[32:30];
				4'd11:	count3	<= count_1[35:33];
				4'd12:	count3	<= count_1[38:36];
				4'd13:	count3	<= count_1[41:39];
				4'd14:	count3	<= count_1[44:42];
				default:count3	<= 0;
				endcase
				if (ptr >= 14)	ptr	<= 0;
				else		ptr	<= ptr + 1;
			end else begin
				count3	<= 0;
				ptr	<= 0;
			end
		end
	end
end
assign	odata_2[2:0]	= count3;

/*
 * Control register 3
 */
always @(posedge clk or negedge rst_) begin
	if (!rst_) begin
		id_0	<= 0;
		id_1	<= 0;
		top	<= `Enable;
		bottom	<= `Enable;
	end else if (sel == 3 && wr) begin
		id_0	<= idata[2:0] * 2 + 0;
		id_1	<= idata[2:0] * 2 + 1;
		top	<= idata[3];
		bottom	<= (idata[2:0] == 0) ? `Enable : `Disable;
	end
end
assign	odata_3[2:0]	= (csel == 0) ? count_0[5:3] :
			  (csel == 1) ? count_1[5:3] : 0;

/*
 * Point-to-point and sbus switch 
 */
assign	c0_idata 	= (mode_0 == `Disable) ? r0_odata_0  : sc_odata;
assign	c0_ivalid 	= (mode_0 == `Disable) ? r0_ovalid_0 : sc_ovalid;
assign	c0_ivch 	= (mode_0 == `Disable) ? r0_ovch_0   : `VCH_P1'b0;
assign	c0_irdy 	= (mode_0 == `Disable) ? r0_ordy_0   : sc_ordy;

assign	c1_idata 	= r1_odata_0;
assign	c1_ivalid 	= r1_ovalid_0;
assign	c1_ivch 	= r1_ovch_0;
assign	c1_irdy 	= r1_ordy_0;

assign	r0_idata_0	= (mode_0 == `Disable) ? c0_odata  : `DATAW_P1'b0;
assign	r0_ivalid_0	= (mode_0 == `Disable) ? c0_ovalid : `Disable;
assign	r0_ivch_0	= (mode_0 == `Disable) ? c0_ovch   : `VCH_P1'b0;

assign	r1_idata_0	= (mode_0 == `Disable) ? c1_odata  : `DATAW_P1'b0;
assign	r1_ivalid_0	= (mode_0 == `Disable) ? c1_ovalid : `Disable;
assign	r1_ivch_0	= (mode_0 == `Disable) ? c1_ovch   : `VCH_P1'b0;

assign	sc_idata	= (mode_0 == `Enable)  ? c0_odata  : `DATAW_P1'b0;
assign	sc_ivalid	= (mode_0 == `Enable)  ? c0_ovalid : `Disable;
assign	sc_irdy		= (mode_0 == `Enable)  ? c0_ovch   : `VCH_P1'b0;

/*
 * Pulse signal generated when my time-slot starts (Sbus only)
 */
assign	ts_start	= (mode_0 == `MODE_P2P) ? `Enable :
			  (mode_0 == `MODE_SBUS && ts_control_last == `Disable) ? ts_control : `Disable;
always @ (posedge clk or negedge rst_) begin
	if (!rst_)
		ts_control_last	<= `Disable;
	else if (ts_control != ts_control_last)
		ts_control_last	<= ts_control;
end

/*
 * Core 0
 */
core #( CHIPID * 2 + 0 ) c0 (
	.ordy   ( ordy_0      ),
	.inject ( inject_0    ),
	.dest   ( dest_0      ),
	.vch    ( vch_0       ),

	.odata  ( c0_odata    ),
	.ovalid ( c0_ovalid   ),
	.ovch   ( c0_ovch     ),
	.irdy   ( c0_irdy     ),

	.clear  ( clear_0     ),
	.count  ( count_0     ),
	.idata  ( c0_idata    ),
	.ivalid ( c0_ivalid   ),

	.slot	( ts_start    ),

	.clk    ( clk         ),
	.rst_   ( rst_        )
);
`ifdef __SIM__
always @(posedge clk) begin
	if (c0_ovalid) begin
		$write("%10d \t Core %3d injects \t %x (vc %d) \n", count, CHIPID * 2 + 0, c0_odata, c0_ovch);
	end
end
always @(posedge clk) begin
	if (c0_ivalid) begin
		$write("%10d \t Core %3d receives \t %x (vc %d) \n", count, CHIPID * 2 + 0, c0_idata, c0_ivch);
	end
end
`endif /* __SIM__ */

/*
 * Core 1
 */
core #( CHIPID * 2 + 1 ) c1 (
	.ordy   ( ordy_1      ),
	.inject ( inject_1    ),
	.dest   ( dest_1      ),
	.vch    ( vch_1       ),

	.odata  ( c1_odata    ),
	.ovalid ( c1_ovalid   ),
	.ovch   ( c1_ovch     ),
	.irdy   ( c1_irdy     ),

	.clear  ( clear_1     ),
	.count  ( count_1     ),
	.idata  ( c1_idata    ),
	.ivalid ( c1_ivalid   ),

	.slot	( `Enable     ),

	.clk    ( clk         ),
	.rst_   ( rst_        )
);
`ifdef __SIM__
always @(posedge clk) begin
	if (c1_ovalid) begin
		$write("%10d \t Core %3d injects \t %x (vc %d) \n", count, CHIPID * 2 + 1, c1_odata, c1_ovch);
	end
end
always @(posedge clk) begin
	if (c1_ivalid) begin
		$write("%10d \t Core %3d receives \t %x (vc %d) \n", count, CHIPID * 2 + 1, c1_idata, c1_ivch);
	end
end
`endif /* __SIM__ */

/*
 * Router 0 (Bubble, downstream)
 */
router #( CHIPID * 2 + 0 ) r0 (
	/* From core */
        .idata_0  ( r0_idata_0  ),
        .ivalid_0 ( r0_ivalid_0 ),
        .ivch_0   ( r0_ivch_0   ),
	.oack_0   ( /* NC */    ),
        .ordy_0   ( r0_ordy_0   ),
	/* To core */
        .odata_0  ( r0_odata_0  ),
        .ovalid_0 ( r0_ovalid_0 ),
        .ovch_0   ( r0_ovch_0   ),
        .iack_0   ( `VCH_P1'b11 ),	/* Core can receive flits at any time */

	/* From horizontal router */
        .idata_1  ( r1_odata_1  ),
        .ivalid_1 ( r1_ovalid_1 ),
        .ivch_1   ( r1_ovch_1   ),
        .oack_1   ( r0_oack_1   ),
        .ordy_1   ( /* NC */    ),
	/* To horizontal router */
        .odata_1  ( r0_odata_1  ),
        .ovalid_1 ( r0_ovalid_1 ),
        .ovch_1   ( r0_ovch_1   ),
        .iack_1   ( r1_oack_1   ),

	/* From vertical (from upper to here) */
        .idata_2  ( r0_idata_2  ),
        .ivalid_2 ( r0_ivalid_2 ),
        .ivch_2   ( r0_ivch_2   ),
        .oack_2   ( r0_oack_2   ),
        .ordy_2   ( /* NC */    ),
	/* To vertical (from here to lower) */
        .odata_2  ( r0_odata_2  ),
        .ovalid_2 ( /* NC */    ),
        .ovch_2   ( r0_ovch_2   ),
        .iack_2   ( r0_iack_2   ),

        .id       ( id_0        ),
        .top      ( top         ),
        .bottom   ( bottom      ),

        .clk      ( clk         ),
        .rst_     ( rst_        )
);
`ifdef __SIM__
always @(posedge clk) begin
	if (r0_ovalid_1) begin
		$write("%10d \t\t Router %3d forwards \t %x to Hrouter (vc %d) \n", count, CHIPID * 2 + 0, r0_odata_1, r0_ovch_1);
	end
end
always @(posedge clk) begin
	if (r0_odata_2[`TYPE_MSB:`TYPE_LSB] > 0) begin
		$write("%10d \t\t Router %3d forwards \t %x to Vrouter (vc %d) \n", count, CHIPID * 2 + 0, r0_odata_2, r0_ovch_2);
	end
end
`endif /* __SIM__ */

/*
 * Router 1 (Bubble, upstream)
 */
router #( CHIPID * 2 + 1 ) r1 (
	/* From core */
        .idata_0  ( r1_idata_0  ),
        .ivalid_0 ( r1_ivalid_0 ),
        .ivch_0   ( r1_ivch_0   ),
	.oack_0   ( /* NC */    ),
        .ordy_0   ( r1_ordy_0   ),
	/* To core */
        .odata_0  ( r1_odata_0  ),
        .ovalid_0 ( r1_ovalid_0 ),
        .ovch_0   ( r1_ovch_0   ),
        .iack_0   ( `VCH_P1'b11 ),	/* Core can receive flits at any time */

	/* From horizontal router */
        .idata_1  ( r0_odata_1  ),
        .ivalid_1 ( r0_ovalid_1 ),
        .ivch_1   ( r0_ovch_1   ),
        .oack_1   ( r1_oack_1   ),
        .ordy_1   ( /* NC */    ),
	/* To horizontal router */
        .odata_1  ( r1_odata_1  ),
        .ovalid_1 ( r1_ovalid_1 ),
        .ovch_1   ( r1_ovch_1   ),
        .iack_1   ( r0_oack_1   ),

	/* From vertical (from lower to here) */
        .idata_2  ( r1_idata_2  ),
        .ivalid_2 ( r1_ivalid_2 ),
        .ivch_2   ( r1_ivch_2   ),
        .oack_2   ( r1_oack_2   ),
        .ordy_2   ( /* NC */    ),
	/* To vertical (from here to upper) */
        .odata_2  ( r1_odata_2  ),
        .ovalid_2 ( /* NC */    ),
        .ovch_2   ( r1_ovch_2   ),
        .iack_2   ( r1_iack_2   ),

        .id       ( id_1        ),
        .top      ( top         ),
        .bottom   ( bottom      ),

        .clk      ( clk         ),
        .rst_     ( rst_        )
);
`ifdef __SIM__
always @(posedge clk) begin
	if (r1_ovalid_1) begin
		$write("%10d \t\t Router %3d forwards \t %x to Hrouter (vc %d) \n", count, CHIPID * 2 + 1, r1_odata_1, r1_ovch_1);
	end
end
always @(posedge clk) begin
	if (r1_odata_2[`TYPE_MSB:`TYPE_LSB] > 0) begin
		$write("%10d \t\t Router %3d forwards \t %x to Vrouter (vc %d) \n", count, CHIPID * 2 + 1, r1_odata_2, r1_ovch_2);
	end
end
`endif /* __SIM__ */

/*
 * Shared bus 0
 */
sbus #( CHIPID ) s0 (
	// Input from Core 0
	.idata_0   ( sc_idata   ),
	.ivalid_0  ( sc_ivalid  ),
	.irdy_0    ( sc_irdy    ),
	// Output to Core 0
	.odata_0   ( sc_odata   ),
	.ovalid_0  ( sc_ovalid  ),
	.ordy_0    ( sc_ordy    ),

	// Input from wireless
	.idata_1  ( id_0[2:1] == 0 ? sidata_0 :
		    id_0[2:1] == 1 ? sidata_1 :
		    id_0[2:1] == 2 ? sidata_2 :
		    id_0[2:1] == 3 ? sidata_3 : `TRDATAW'h0 ),
	// Output to wireless
	.odata_1   ( str_odata  ),

	.sel       ( str_sel    ),

	.id        ( id_0       ),
	.top       ( top        ),
	.bottom    ( bottom     ),

	.ts_control( ts_control ),  			     

	.clk       ( clk        ),
	.rst_      ( rst_       )
);
assign	sodata_0 = ( id_0[2:1] == 0 ) ? str_odata : `TRDATAW'h0;
assign	sodata_1 = ( id_0[2:1] == 1 ) ? str_odata : `TRDATAW'h0;
assign	sodata_2 = ( id_0[2:1] == 2 ) ? str_odata : `TRDATAW'h0;
assign	sodata_3 = ( id_0[2:1] == 3 ) ? str_odata : `TRDATAW'h0;
`ifdef __SIM__
always @(posedge clk) begin
	if (str_odata[`TYPE_MSB:`TYPE_LSB] > 0) begin
		$write("%10d \t\t Sbus %3d forwards \t %x to Vertical \n", count, CHIPID, str_odata);
	end
end
`endif /* __SIM__ */

/*
 * TX/RX configuration
 */
/* From upper to here (Router 0) */ 
assign	bsel_0		= (top || mode_0 != `MODE_P2P) ? `TRSEL_IDLE  : 
			  (dummy == `Enable && id_0 == 0) ? `TRSEL_TX :	/* Dummy */
				`TRSEL_RX;
assign	r0_idata_2	= (top || mode_0 != `MODE_P2P) ? `DATAW_P1'b0 : 
				bidata_0[`DATAW:0];
assign	r0_ivch_2	= (top || mode_0 != `MODE_P2P) ? `VCHW_P1'b0  : 
				bidata_0[`TRDATAW:`DATAW_P1];
assign	r0_ivalid_2	= (top || mode_0 != `MODE_P2P) ? `Disable     : 
				r0_idata_2[`TYPE_MSB:`TYPE_LSB] > 0;
assign	bsel_1		= (top || mode_0 != `MODE_P2P) ? `TRSEL_IDLE  : 
			  (dummy == `Enable && id_0 == 2) ? `TRSEL_RX :	/* Dummy */
				`TRSEL_TX;
assign	bodata_1	= (top || mode_0 != `MODE_P2P) ? `VCH_P1'b0   : 
				r0_oack_2;

/* From here to lower (Router 0) */ 
assign	bsel_2		= (bottom || mode_0 != `MODE_P2P) ? `TRSEL_IDLE    : 
			  (dummy == `Enable && id_0 == 4) ? `TRSEL_RX :	/* Dummy */
				`TRSEL_TX;
assign	bodata_2	= (bottom || mode_0 != `MODE_P2P) ? `TRDATAW_P1'b0 : 
				{r0_ovch_2, r0_odata_2};
assign	bsel_3		= (bottom || mode_0 != `MODE_P2P) ? `TRSEL_IDLE    : 
			  (dummy == `Enable && id_0 == 6) ? `TRSEL_TX :	/* Dummy */
				`TRSEL_RX;
assign	r0_iack_2	= (bottom || mode_0 != `MODE_P2P) ? `VCH_P1'b0     : 
				bidata_3;

/* From here to upper (Router 1) */ 
assign	bsel_4		= (top || mode_0 != `MODE_P2P) ? `TRSEL_IDLE    : 
			  (dummy == `Enable && id_0 != 2) ? `TRSEL_RX :	/* Dummy */
				`TRSEL_TX;
assign	bodata_4	= (top || mode_0 != `MODE_P2P) ? `TRDATAW_P1'b0 : 
				{r1_ovch_2, r1_odata_2};
assign	bsel_5		= (top || mode_0 != `MODE_P2P) ? `TRSEL_IDLE    : 
			  (dummy == `Enable && id_0 != 0) ? `TRSEL_TX :	/* Dummy */
				`TRSEL_RX;
assign	r1_iack_2	= (top || mode_0 != `MODE_P2P) ? `VCH_P1'b0     : 
				bidata_5;

/* From lower to here (Router 1) */ 
assign	bsel_6		= (bottom || mode_0 != `MODE_P2P) ? `TRSEL_IDLE  : 
			  (dummy == `Enable && id_0 != 6) ? `TRSEL_TX :	/* Dummy */
				`TRSEL_RX;
assign	r1_idata_2	= (bottom || mode_0 != `MODE_P2P) ? `DATAW_P1'b0 : 
				bidata_6[`DATAW:0];
assign	r1_ivch_2	= (bottom || mode_0 != `MODE_P2P) ? `VCHW_P1'b0  : 
				bidata_6[`TRDATAW:`DATAW_P1];
assign	r1_ivalid_2	= (bottom || mode_0 != `MODE_P2P) ? `Disable     : 
				r1_idata_2[`TYPE_MSB:`TYPE_LSB] > 0;
assign	bsel_7		= (bottom || mode_0 != `MODE_P2P) ? `TRSEL_IDLE  : 
			  (dummy == `Enable && id_0 != 4) ? `TRSEL_RX :	/* Dummy */
				`TRSEL_TX;
assign	bodata_7	= (bottom || mode_0 != `MODE_P2P) ? `VCH_P1'b0   : 
				r1_oack_2;

/* Sbus */
assign	ssel_0 		= (id_0[2:1] == 0 && mode_0 == `MODE_SBUS) ? str_sel :
				`TRSEL_IDLE;
assign	ssel_1 		= (id_0[2:1] == 1 && mode_0 == `MODE_SBUS) ? str_sel :
				`TRSEL_IDLE;
assign	ssel_2 		= (id_0[2:1] == 2 && mode_0 == `MODE_SBUS) ? str_sel :
				`TRSEL_IDLE;
assign	ssel_3 		= (id_0[2:1] == 3 && mode_0 == `MODE_SBUS) ? str_sel :
				`TRSEL_IDLE;

/*
 * Probe signal for top chip
 */
/* Core 0 CHIPID */ 
assign	probe[1:0]	= id_0[2:1];
/* My timeslot (sbus mode only) */ 
assign	probe[2]	= ts_control;
/* Core 0 --> Router 0 */
assign	probe[4:3]	= c0_odata[`TYPE_MSB:`TYPE_LSB];
/* Router 0 --> Router 1 */
assign	probe[6:5]	= r0_odata_1[`TYPE_MSB:`TYPE_LSB];
/* Router 1 --> Core 1 */
assign	probe[8:7]	= r1_odata_0[`TYPE_MSB:`TYPE_LSB];

endmodule
