/* $Id: inputc.v 105 2010-08-12 09:00:48Z matutani $ */
`include "../design/define.h" 
module inputc ( 
        idata,  
        ivalid, 
        ivch,   
        oack,   
        ordy,   

        odata,  
        ovalid, 
        ovch,   

        irdy0_0, 
        irdy0_1, 
        irdy0_2, 
        irdy1_0, 
        irdy1_1, 
        irdy1_2, 

        grt_0, 
        grt_1, 
        grt_2, 

        port,  
        req,   

        id, 
	top,
	bottom,

        clk, 
        rst_
);
parameter       ROUTERID= 0;
parameter       PCHID   = 0;

input   [`DATAW:0]      idata;  
input                   ivalid; 
input   [`VCHW:0]       ivch;   
output  [`VCH:0]        oack;   
output  [`VCH:0]        ordy;   

output  [`DATAW:0]      odata;  
output                  ovalid; 
output  [`VCHW:0]       ovch;   

input   [`VCH:0]        irdy0_0; 
input   [`VCH:0]        irdy0_1; 
input   [`VCH:0]        irdy0_2; 
input   [`VCH:0]        irdy1_0; 
input   [`VCH:0]        irdy1_1; 
input   [`VCH:0]        irdy1_2; 

input                   grt_0; 
input                   grt_1; 
input                   grt_2; 

output  [`PORTW:0]      port;   
output                  req;    

input	[`NODEW:0]	id;
input			top;
input			bottom;

input   clk, rst_;               

wire	[`TYPEW:0]	itype;

wire	[`DATAW:0]      odata0;  
wire                    ovalid0; 
wire    [`VCHW:0]       ovch0;  
wire    [`PORTW:0]      port0;  
wire                    req0;   
wire			rd0;

wire	[`DATAW:0]      odata1;  
wire                    ovalid1; 
wire    [`VCHW:0]       ovch1;  
wire    [`PORTW:0]      port1;  
wire                    req1;   
wire			rd1;

wire    [`VCH:0]        vcsel;

wire    [`PORTW:0]      rc_port0; 
wire    [`VCHW:0]       rc_ovch0; 
wire	[`DATAW:0]      vc_idata0;  
wire			vc_ivalid0;
wire    [`DATAW:0]      bdata0; 
wire    [`TYPEW:0]      btype0; 

wire    [`PORTW:0]      rc_port1; 
wire    [`VCHW:0]       rc_ovch1; 
wire	[`DATAW:0]      vc_idata1;  
wire			vc_ivalid1;
wire    [`DATAW:0]      bdata1; 
wire    [`TYPEW:0]      btype1; 

/* 
 * Input buffers 
 */ 
fifo    ibuf0 ( 
        .idata  ( ivch == 0 ? idata  : `DATAW_P1'b0 ), 
        .odata  ( bdata0 ), 

        .wr_en  ( ivch == 0 ? ivalid : `Disable ), 
        .rd_en  ( rd0 ), 

//	.empty  ( /* not used */ ), 
//	.full   ( /* not used */ ), 

        .ordy   ( ordy[0]        ), 

        .clk    ( clk  ), 
        .rst_   ( rst_ )  
); 
/*
always @(posedge clk) begin
	if (ibuf0.d_cnt > 10 ) begin
		$write("XXX cnt %d router %d pch %d vch %d \n", ibuf0.d_cnt, ROUTERID, PCHID, 0);
	end
end
*/

fifo    ibuf1 ( 
        .idata  ( ivch == 1 ? idata  : `DATAW_P1'b0 ), 
        .odata  ( bdata1 ), 

        .wr_en  ( ivch == 1 ? ivalid : `Disable ), 
        .rd_en  ( rd1 ), 

//	.empty  ( /* not used */ ), 
//	.full   ( /* not used */ ), 

        .ordy   ( ordy[1]        ), 

        .clk    ( clk  ), 
        .rst_   ( rst_ )  
); 

/* 
 * Routing computation logic 
 */ 
rtcomp #( ROUTERID, PCHID, 0 ) rc0 ( 
        .addr   ( vc_idata0[`DST_MSB:`DST_LSB] ),
        .port   ( rc_port0 ),
        .ovch   ( rc_ovch0 ),

        .id     ( id       ),
        .top    ( top      ),
        .bottom ( bottom   )

//      .clk    ( clk      ),
//      .rst_   ( rst_     ) 
);

rtcomp #( ROUTERID, PCHID, 1 ) rc1 ( 
        .addr   ( vc_idata1[`DST_MSB:`DST_LSB] ),
        .port   ( rc_port1 ),
        .ovch   ( rc_ovch1 ),

        .id     ( id       ),
        .top    ( top      ),
        .bottom ( bottom   )

//      .clk    ( clk      ),
//      .rst_   ( rst_     ) 
);

/* 
 * Input virtual channels 
 */ 
vc #( ROUTERID, PCHID, 0 ) vc_0 ( 
       .idata  ( vc_idata0  ), 
       .ivalid ( vc_ivalid0 ), 

       .odata  ( odata0     ), 
       .ovalid ( ovalid0    ), 

       .rd     ( rd0        ), 

       .irdy0_0( irdy0_0    ), 
       .irdy0_1( irdy0_1    ), 
       .irdy0_2( irdy0_2    ), 
       .irdy1_0( irdy1_0    ), 
       .irdy1_1( irdy1_1    ), 
       .irdy1_2( irdy1_2    ), 

       .grt_0  ( vcsel[0] ? grt_0 : `Disable ), 
       .grt_1  ( vcsel[0] ? grt_1 : `Disable ), 
       .grt_2  ( vcsel[0] ? grt_2 : `Disable ), 

       .port   ( port0      ), 
       .ovch   ( ovch0      ), 
       .req    ( req0       ), 

       .rc_port( rc_port0   ),
       .rc_ovch( rc_ovch0   ),

       .top    ( top        ), 
       .bottom ( bottom     ), 

       .clk    ( clk        ), 
       .rst_   ( rst_       )
);                  

vc #( ROUTERID, PCHID, 1 ) vc_1 ( 
       .idata  ( vc_idata1  ), 
       .ivalid ( vc_ivalid1 ), 

       .odata  ( odata1     ), 
       .ovalid ( ovalid1    ), 

       .rd     ( rd1        ), 

       .irdy0_0( irdy0_0    ), 
       .irdy0_1( irdy0_1    ), 
       .irdy0_2( irdy0_2    ), 
       .irdy1_0( irdy1_0    ), 
       .irdy1_1( irdy1_1    ), 
       .irdy1_2( irdy1_2    ), 

       .grt_0  ( vcsel[1] ? grt_0 : `Disable ), 
       .grt_1  ( vcsel[1] ? grt_1 : `Disable ), 
       .grt_2  ( vcsel[1] ? grt_2 : `Disable ), 

       .port   ( port1      ), 
       .ovch   ( ovch1      ), 
       .req    ( req1       ), 

       .rc_port( rc_port1   ),
       .rc_ovch( rc_ovch1   ),

       .top    ( top        ), 
       .bottom ( bottom     ), 

       .clk    ( clk        ),
       .rst_   ( rst_       )
);                  

/* 
 * VC mux 
 */ 
vcmux vcmux ( 
        .ovalid0 ( ovalid0 ),
        .ovalid1 ( ovalid1 ),

        .odata0  ( odata0  ),
        .odata1  ( odata1  ),

        .ovch0   ( ovch0   ),
        .ovch1   ( ovch1   ),

        .req0    ( req0    ),
        .req1    ( req1    ),

        .port0   ( port0   ),
        .port1   ( port1   ),

        .ovalid  ( ovalid  ),
        .odata   ( odata   ),
        .ovch    ( ovch    ),
        .req     ( req     ),
        .port    ( port    ),
        .vcsel   ( vcsel   ),

        .clk     ( clk     ),
        .rst_    ( rst_    ) 
);

/*
 * Input flit type
 */
assign  itype   = (ivalid == `Enable) ? idata[`TYPE_MSB:`TYPE_LSB] :
                                        `TYPE_NONE;

/*
 * Incoming flit for each VC (from buffer or network)
 */
assign	vc_idata0	= (btype0 > 0)              ? bdata0 : 
			  (itype  > 0 && ivch == 0) ? idata  : `DATAW_P1'b0;
assign	vc_idata1	= (btype1 > 0)              ? bdata1 : 
			  (itype  > 0 && ivch == 1) ? idata  : `DATAW_P1'b0;
assign	vc_ivalid0	= (btype0 > 0) || (itype > 0 && ivch == 0);
assign	vc_ivalid1	= (btype1 > 0) || (itype > 0 && ivch == 1);

/*
 * Credit back to the upstream router
 */
assign	oack[0]	= (ovalid0 && odata0[`TYPE_MSB:`TYPE_LSB] == `TYPE_TAIL);
assign	oack[1]	= (ovalid1 && odata1[`TYPE_MSB:`TYPE_LSB] == `TYPE_TAIL);

/*
 * Buffered flit type
 */
assign  btype0  = bdata0[`TYPE_MSB:`TYPE_LSB]; 
assign  btype1  = bdata1[`TYPE_MSB:`TYPE_LSB]; 

endmodule
