/* $Id: outputc.v 27 2010-07-25 09:11:12Z matutani $ */
`include "../design/define.h" 
module outputc (
        idata,
        ivalid,
        ivch,

        odata,
        ovalid,
        ovch,

        iack,
        ordy0,
        ordy1,

        clk,
        rst_
);
parameter       ROUTERID= 0;
parameter       PCHID   = 0;

input   [`DATAW:0]      idata;
input                   ivalid;
input   [`VCHW:0]       ivch;

output  [`DATAW:0]      odata;
output                  ovalid;
output  [`VCHW:0]       ovch;

input   [`VCH:0]        iack;
output  [`VCH:0]        ordy0;	/* Ready for injection from ring */
output  [`VCH:0]        ordy1;	/* Ready for injection from core */

input   clk, rst_;

reg     [`DATAW:0]      odata;
reg                     ovalid;
reg     [`VCHW:0]       ovch;

wire    [`TYPEW:0]      itype;

reg     [`FIFOD_P1:0]   cnt0; 	/* XXX: 2-bit is enough */
reg     [`FIFOD_P1:0]   cnt1; 	/* XXX: 2-bit is enough */

/*
 * Input flit type
 */
assign  itype   = (ivalid == `Enable) ? idata[`TYPE_MSB:`TYPE_LSB] :
                                        `TYPE_NONE;

/*
 * Output data
 */
always @ (posedge clk or negedge rst_) begin
        if (!rst_) begin
                odata   <= `DATAW_P1'b0;
                ovalid  <= `Disable;
                ovch    <= `VCHW_P1'b0;
        end else if (ivalid) begin
                odata   <= idata;
                ovalid  <= ivalid;
                ovch    <= ivch;
        end else if (~ivalid & ovalid) begin
                odata   <= `DATAW_P1'b0;
                ovalid  <= `Disable;
                ovch    <= `VCHW_P1'b0;
        end
end

/*
always @(posedge clk) begin
	if (cnt0 > 1) $write("---- VCT debug router %d port %d cnt %d \n", ROUTERID, PCHID, cnt0);
end
*/

/*  
 * Buffer occupancy of the downstream router 
 */ 
/* Ready for a packet injection from the same dimension */
assign  ordy0[0] = (cnt0 <= 2);
assign  ordy0[1] = (cnt1 <= 2);

/* Ready for a packet injection from different dimension (or core) */
assign  ordy1[0] = (cnt0 <= 1);
assign  ordy1[1] = (cnt1 <= 1);

always @ (posedge clk or negedge rst_) begin                      
        if (!rst_)                                  
                cnt0  <= 0;                      
        else if (iack[0] && ~(itype == `TYPE_HEAD && ivch == 0) && cnt0 > 0) 
                cnt0  <= cnt0 - 1;              
        else if (~iack[0] && (itype == `TYPE_HEAD && ivch == 0)) 
                cnt0  <= cnt0 + 1;              
end                                               
always @ (posedge clk or negedge rst_) begin                      
        if (!rst_)                                  
                cnt1  <= 0;                      
        else if (iack[1] && ~(itype == `TYPE_HEAD && ivch == 1) && cnt1 > 0) 
                cnt1  <= cnt1 - 1;              
        else if (~iack[1] && (itype == `TYPE_HEAD && ivch == 1)) 
                cnt1  <= cnt1 + 1;              
end                                               

endmodule 
