/* $Id: sbus.v 84 2010-08-10 08:35:49Z sasaki $ */
`include "../design/define.h"
module sbus (
        idata_0,
        ivalid_0,
        irdy_0,

        odata_0,
        ovalid_0,
        ordy_0,

        idata_1,
        odata_1,

	sel,

        id,
        top,
	bottom,

	ts_control,
	     
	clk,
	rst_
	
);
    
parameter		SBUSID	= 0;

input	[`DATAW:0]	idata_0;	/* Input from Core */
input			ivalid_0;
input	[`VCH:0]	irdy_0;

output	[`DATAW:0]	odata_0;	/* Output to Core */
output			ovalid_0;
output	[`VCH:0]	ordy_0;
   
input	[`TRDATAW:0]	idata_1;	/* Input from 0-Vertical (from RX) */
output	[`TRDATAW:0]	odata_1;        /* Output to 0-Vertical (to TX) */

output  [`TRSELW:0]     sel;
    
input	[`NODEW:0]	id;
input			top;
input			bottom;

 
output                  ts_control;
    
input			clk, rst_;
    
reg     [4:0]           count;

reg     [`DATAW:0]      odata_0;
reg                     ovalid_0;
reg     [`VCH:0]        ordy_0; 

reg     [`TRDATAW:0]    odata_1;

reg                     ts_control;
reg     [`TRSELW:0]     sel;

reg                     stop;
    
wire     [`DATAW:0]      c_data;
wire                     c_valid;
wire     [`VCH:0]        c_rdy; 

wire     [`TRDATAW:0]    data;

wire                     ts;
wire     [`TRSELW:0]     sel_t;

/*
 * Data transmission (Core --> TX)
 */
always @ (posedge clk or negedge rst_) begin
	if ( !rst_ ) begin
	    count    <= 5'b0;
	    stop     <= `Disable;
	end else begin
	    count    <= count + 5'b1;
	    if ( count == 10 && !stop) begin
		stop <= `Enable;
	    end
	end
end
    
assign c_rdy  = ((count == 31) && (id[2:1] == 0) ) ? 2'h1 :
		((count[4:3] + 1 == id[2:1]) && (count[2:0] == 7)) ? 2'h1 :
		( count[4:3] != id[2:1]) ? 2'h0 :
		(( count[4:3] == id[2:1]) && (count[2:0] >= 5)) ? 2'h0 : 2'h1;
/*
assign c_data   = ( !stop ) ? `DATAW_P1'h0 :
		  ( ivalid_0 && ( idata_0[`NODEW:0] == id )) ? idata_0 :
		  (( id[2:1] == 3 ) && ( count[4:3] == 0 ) && (count[2:0] <= 3 )) ? `DATAW_P1'h0 :
		  (( count[4:3] == 0 ) && ( count[2:0] >= 2 && count[2:0] <= 3 )) ? `DATAW_P1'h0 :
		  (( count[4:3] == id[2:1] )&& ( count[2:0] >= 2 )) ? `DATAW_P1'h0 :
		  (( count[4:3] - 1 == id[2:1] ) && ( count[2:0] < 3 )) ? `DATAW_P1'h0 :
		  (( idata_1[`NODEW:0] == id ) && ( idata_1[`TYPE_MSB:`TYPE_LSB] > 0 ) && 
		   ( count[4:3] != id[2:1]  || (count[4:3] == id[2:1] && count[2:0] <= 1 ))) ? idata_1[`DATAW:0] : `DATAW_P1'h0; 
*/
assign c_data   = ( !stop ) ? `DATAW_P1'h0 :
		  ( ivalid_0 && ( idata_0[`NODEW:0] == id )) ? idata_0 :
		  (( id[2:1] == 3 ) && ( count[4:3] == 0 ) && (count[2:0] <= 3 )) ? `DATAW_P1'h0 :
		  (( count[4:3] == 0 ) && ( count[2:0] >= 2 && count[2:0] <= 3 )) ? `DATAW_P1'h0 :
		  (( count[4:3] == id[2:1] )&& ( count[2:0] >= 2 )) ? `DATAW_P1'h0 :
		  (( count[4:3] - 1 == id[2:1] ) && ( count[2:0] < 3 )) ? `DATAW_P1'h0 :
		  (( count[2:0] == 3 )) ? `DATAW_P1'h0 :	// Guard cycle (All chips are in RX mode)
		  (( idata_1[`NODEW:0] == id ) && ( idata_1[`TYPE_MSB:`TYPE_LSB] > 0 ) && 
		   ( count[4:3] != id[2:1]  || (count[4:3] == id[2:1] && count[2:0] <= 1 ))) ? idata_1[`DATAW:0] : `DATAW_P1'h0; 
/*
assign c_valid  = ( !stop ) ? `DATAW_P1'h0 :
		  ( ivalid_0 && ( idata_0[`NODEW:0] == id ) && ( idata_0[`TYPE_MSB:`TYPE_LSB] > 0)) ? `Enable :
		  (( id[2:1] == 3 ) && ( count[4:3] == 0 ) && (count[2:0] <= 3 )) ? `Disable :
		  (( count[4:3] == 0 ) && ( count[2:0] >= 2 && count[2:0] <= 3 )) ? `Disable :
		  (( count[4:3] == id[2:1] )&& ( count[2:0] >= 2 )) ? `Disable :
		  (( (count[4:3] - 1) == id[2:1] ) && ( count[2:0] < 3 )) ? `Disable :
		  (( idata_1[`NODEW:0] == id ) && ( idata_1[`TYPE_MSB:`TYPE_LSB] > 0 ) && 
		   ( count[4:3] != id[2:1] || (count[4:3] == id[2:1] && count[2:0] <= 1 ))) ? `Enable : `Disable;
*/
assign c_valid  = ( !stop ) ? `DATAW_P1'h0 :
		  ( ivalid_0 && ( idata_0[`NODEW:0] == id ) && ( idata_0[`TYPE_MSB:`TYPE_LSB] > 0)) ? `Enable :
		  (( id[2:1] == 3 ) && ( count[4:3] == 0 ) && (count[2:0] <= 3 )) ? `Disable :
		  (( count[4:3] == 0 ) && ( count[2:0] >= 2 && count[2:0] <= 3 )) ? `Disable :
		  (( count[4:3] == id[2:1] )&& ( count[2:0] >= 2 )) ? `Disable :
		  (( (count[4:3] - 1) == id[2:1] ) && ( count[2:0] < 3 )) ? `Disable :
		  (( count[2:0] == 3 )) ? `Disable :	// Guard cycle (All chips are in RX mode)
		  (( idata_1[`NODEW:0] == id ) && ( idata_1[`TYPE_MSB:`TYPE_LSB] > 0 ) && 
		   ( count[4:3] != id[2:1] || (count[4:3] == id[2:1] && count[2:0] <= 1 ))) ? `Enable : `Disable;
assign data     = ( ivalid_0 && ( count[4:3] == id[2:1] ) && ( count[2:0] > 0 && count[2:0] < 7 )) ? {1'b0, idata_0} : `DATAW_P1'h0;
assign ts = (count[4:3] == id[2:1]) ? `Enable : `Disable;
/*
assign sel_t = ( count[4:3] == id[2:1] && count[2:0] >= 2 ) ? `TRSEL_TX : 
	       (( count[4:3] - 1 == id[2:1] ) && count[2:0] <= 1 ) ? `TRSEL_TX : `TRSEL_RX;
*/
assign sel_t = ( count[4:3] == id[2:1] && count[2:0] >= 2 ) ? `TRSEL_TX : 
	       (( count[4:3] - 1 == id[2:1] ) && id[2:1] != 3 && count[2:0] <= 0 ) ? `TRSEL_TX :
	       (( count[4:3]     == 0       ) && id[2:1] == 3 && count[2:0] <= 0 ) ? `TRSEL_TX : `TRSEL_RX;
	      

always @ (posedge clk or negedge rst_) begin
    if ( !rst_ ) begin
        odata_0  <= `DATAW'h0;
	ovalid_0 <= `Disable;
	ordy_0   <= `VCH'h0;
	odata_1  <= `TRDATAW'h0;
	ts_control <= `Disable;
	sel      <= `TRSEL_IDLE;
    end else begin
      	if ( c_valid ) begin
	    odata_0  <= c_data;
	    ovalid_0 <= c_valid;
	end else if ( ovalid_0 ) begin
	    odata_0  <= `DATAW'h0;
	    ovalid_0 <= `Disable;
	end	      
	
	if ( data[`TYPE_MSB:`TYPE_LSB] > 0 ) begin
	    odata_1 <= data;
	end else if ( !ivalid_0 && odata_1[`TYPE_MSB:`TYPE_LSB] > 0 ) begin
	    odata_1 <= `TRDATAW'h0;
	end

	ordy_0   <= c_rdy;
	ts_control <= ts;
	sel      <= sel_t;
    end
end
endmodule
