/* $Id: vc.v 28 2010-07-26 17:08:57Z matutani $ */
`include "../design/define.h" 
module vc ( 
        idata,
	ivalid,
        odata,   
	ovalid,

	rd,

        irdy0_0, 
        irdy0_1, 
        irdy0_2, 
        irdy1_0, 
        irdy1_1, 
        irdy1_2, 

        grt_0, 
        grt_1, 
        grt_2, 

        port, 
        ovch, 
        req,  

        rc_port, 
        rc_ovch, 

	top,
	bottom,

        clk, 
        rst_
);
parameter       ROUTERID= 0;
parameter       PCHID   = 0;
parameter       VCHID   = 0;

input	[`DATAW:0]	idata;
input			ivalid;
output	[`DATAW:0]	odata;
output			ovalid;

output			rd;

input   [`VCH:0]        irdy0_0; 
input   [`VCH:0]        irdy0_1; 
input   [`VCH:0]        irdy0_2; 
input   [`VCH:0]        irdy1_0; 
input   [`VCH:0]        irdy1_1; 
input   [`VCH:0]        irdy1_2; 

input                   grt_0; 
input                   grt_1; 
input                   grt_2; 

output  [`PORTW:0]      port;   
output  [`VCHW:0]       ovch;   
output                  req;    

input   [`PORTW:0]      rc_port; 
input   [`VCHW:0]       rc_ovch; 

input			top;
input			bottom;

input   		clk, rst_;               


reg     [1:0]           state;

reg	[`DATAW:0]	odata;	/* Output data */
reg			ovalid;	/* Output data available */
reg     [`PORTW:0]      port;    /* Request port   */
reg     [`VCHW:0]       ovch;    /* Request VC     */
reg                     req;     /* Request signal */

wire                    grt;	 /* Output channel is allocated */
wire			irdy;

wire    [`TYPEW:0]      itype;

/*
 * Input/Output flit type
 */
assign  itype   = (ivalid == `Enable) ? idata[`TYPE_MSB:`TYPE_LSB] :
                                        `TYPE_NONE;

/*
 * State machine
 */
`define RC_STAGE      2'b00
`define VSA_STAGE     2'b01
`define ST_STAGE      2'b10

/*  
 * Crossbar switch is allocated or not 
 */ 
assign  grt     = ( 
                   (port == 0 && grt_0) || 
                   (port == 1 && grt_1) || 
                   (port == 2 && grt_2) ); 

/*
 * Ready for a packet injection
 *   irdy0 for a packet injection from the same dimension
 *   irdy1 for a packet injection from different dimension or core
 */
assign  irdy   = ( 
                   (port == 0 && irdy1_0[ovch]) || 
                   (port == 1 && irdy0_1[ovch] && (bottom || top)) || 
                   (port == 1 && irdy1_1[ovch]) || 
                   (port == 2 && irdy0_2[ovch]) ); 

/*
 * Buffer read enable (Read the next data)
 */
assign	rd	= (state == `VSA_STAGE && irdy && grt) ? `Enable :
		  (state == `ST_STAGE) ? `Enable : `Disable;

/*
 * State transition control
 */
always @ (posedge clk or negedge rst_) begin
	if (!rst_) begin
		state	<= `RC_STAGE;
		port	<= 0;
                ovch    <= 0;
		req	<= `Disable;
		odata	<= 0;
		ovalid	<= `Disable;
	end else begin
		case (state)

		/*
		 * State 1 : Routing computation
		 */
		`RC_STAGE: begin
			if (itype == `TYPE_HEAD) begin
				port	<= rc_port;
				ovch	<= rc_ovch;
				req	<= `Enable;
				odata	<= 0;
				ovalid	<= `Disable;
				state	<= `VSA_STAGE;
			end else if (port | ovch | req | odata | ovalid) begin
				port	<= 0;
				ovch	<= 0;
				req	<= `Disable;
				odata	<= 0;
				ovalid	<= `Disable;
			end
		end

                /*
                 * State 2 : Virtual channel / switch allocation
                 */
                `VSA_STAGE: begin
			if (!irdy) begin
				/*
				 * The next buffer does not have one or 
				 * two packets space.
				 */
				req     <= `Disable;
			end else if (grt) begin
				/*
				 * The next buffer has one or two packets space,
				 * and the switch is allocated for me!
				 */
				odata	<= idata;
				ovalid	<= ivalid;
                                state   <= `ST_STAGE;
			end else begin
				/*
				 * The next buffer has one or two packets space,
				 * but the switch is not allocated.
				 */
				req     <= `Enable;
			end
                end

                /*
                 * State 3 : Switch traversal
                 */
		`ST_STAGE: begin
			odata	<= idata;
			ovalid	<= ivalid;
			if (itype == `TYPE_TAIL) begin
				state	<= `RC_STAGE;
				req	<= `Disable;
			end
		end

		endcase
	end
end

endmodule
