#!/bin/sh

# Number of simulations
SIM=2

# Number of cores
N=8

for I in `seq 1 ${SIM}`; do 

	# Generate the event ROM
	./rom.pl > event.hex

	# Start simulation
	make sim > /dev/null

	# Number of flits sent
	SENT=`cat sim.log | awk '$2=="Total" && $3=="sent" {print $4}'`

	# Number of flits received
	RECV=`cat sim.log | awk '$2=="Total" && $3=="received" {print $4}'`

	# First packet injection time
	STIME=`cat sim.log | awk '$2=="Core" && $4=="injects" {print $1; exit}'`

	# Last packet injection time
	ETIME=`cat sim.log | awk '$2=="Core" && $4=="receives" {ETIME=$1} END {print ETIME}'`

	# Calculate the throughput
	THR=`echo ${N} ${RECV} ${STIME} ${ETIME} | awk '{print $2/($4-$3)/$1}'`

	# Check the numbers of flits sent and received
	if test ${SENT} -eq ${RECV}; then
		echo "${I} CheckOK Sent ${SENT} Recv ${RECV} Throughput ${THR} [flits/core/cycle]"
	else
		echo "${I} Error   Sent ${SENT} Recv ${RECV} Throughput ${THR} [flits/core/cycle]"
		exit
	fi

done

