/* $Id: cube_top.v 88 2010-08-10 10:18:11Z matutani $ */
`include "../design/define.h"
module cube_top (
	idata,
	odata,
	sel,
	wr,

	probe,

	wireless_bidata_0,
	wireless_bodata_1,

	wireless_bodata_2,
	wireless_bidata_3,

	wireless_bodata_4,
	wireless_bidata_5,

	wireless_bidata_6,
	wireless_bodata_7,

	wireless_sidata,
	wireless_sodata,

	clk,
	rst_
);
parameter		CHIPID = 0;

input	[3:0]		idata;
output	[2:0]		odata;
input	[1:0]		sel;
input			wr;

output	[8:0]		probe;

/* From upper to here (Bubble, downstream) */ 
input	[`TRDATAW:0]	wireless_bidata_0;
output	[`TRDATAW:0]	wireless_bodata_1;

/* From here to lower (Bubble, downstream) */ 
output	[`TRDATAW:0]	wireless_bodata_2;
input	[`TRDATAW:0]	wireless_bidata_3;

/* From here to upper (Bubble, upstream) */ 
output	[`TRDATAW:0]	wireless_bodata_4;
input	[`TRDATAW:0]	wireless_bidata_5;

/* From lower to here (Bubble, upstream) */ 
input	[`TRDATAW:0]	wireless_bidata_6;
output	[`TRDATAW:0]	wireless_bodata_7;

/* Sbus */ 
input	[`TRDATAW:0]	wireless_sidata;
output	[`TRDATAW:0]	wireless_sodata;

input			clk;
input			rst_;		/* Active-low */

/* From upper to here (Bubble, downstream) */
wire	[`TRDATAW:0]	bidata_0;
wire	[`TRSELW:0]	bsel_0;
wire	[`VCH:0]	bodata_1;
wire	[`TRSELW:0]	bsel_1;

/* From here to lower (Bubble, downstream) */
wire	[`TRDATAW:0]	bodata_2;
wire	[`TRSELW:0]	bsel_2;
wire	[`VCH:0]	bidata_3;
wire	[`TRSELW:0]	bsel_3;

/* From here to upper (Bubble, upstream) */
wire	[`TRDATAW:0]	bodata_4;
wire	[`TRSELW:0]	bsel_4;
wire	[`VCH:0]	bidata_5;
wire	[`TRSELW:0]	bsel_5;

/* From lower to here (Bubble, upstream) */
wire	[`TRDATAW:0]	bidata_6;
wire	[`TRSELW:0]	bsel_6;
wire	[`VCH:0]	bodata_7;
wire	[`TRSELW:0]	bsel_7;

wire	[(`TRDATAW - `VCH - 1):0]	bzeros;
wire	[(`TRDATAW - `VCH - 1):0]	null0;
wire	[(`TRDATAW - `VCH - 1):0]	null1;

/* TX/RX 0 (Sbus) */
wire	[`TRDATAW:0]	wireless_sidata_0;
wire	[`TRDATAW:0]	wireless_sodata_0;
wire	[`TRDATAW:0]	sidata_0;
wire	[`TRDATAW:0]	sodata_0;
wire	[`TRSELW:0]	ssel_0;

/* TX/RX 1 (Sbus) */
wire	[`TRDATAW:0]	wireless_sidata_1;
wire	[`TRDATAW:0]	wireless_sodata_1;
wire	[`TRDATAW:0]	sidata_1;
wire	[`TRDATAW:0]	sodata_1;
wire	[`TRSELW:0]	ssel_1;

/* TX/RX 2 (Sbus) */
wire	[`TRDATAW:0]	wireless_sidata_2;
wire	[`TRDATAW:0]	wireless_sodata_2;
wire	[`TRDATAW:0]	sidata_2;
wire	[`TRDATAW:0]	sodata_2;
wire	[`TRSELW:0]	ssel_2;

/* TX/RX 3 (Sbus) */
wire	[`TRDATAW:0]	wireless_sidata_3;
wire	[`TRDATAW:0]	wireless_sodata_3;
wire	[`TRDATAW:0]	sidata_3;
wire	[`TRDATAW:0]	sodata_3;
wire	[`TRSELW:0]	ssel_3;

/*
 * Cube chip
 */
`ifdef __SIM__
cube #( CHIPID ) c0 (
`else
cube             c0 (
`endif
	.idata    ( idata    ),
	.odata    ( odata    ),
	.sel      ( sel      ),
	.wr       ( wr       ),

	.probe    ( probe    ),

	.bidata_0 ( bidata_0 ),
	.bsel_0   ( bsel_0   ),
	.bodata_1 ( bodata_1 ),
	.bsel_1   ( bsel_1   ),

	.bodata_2 ( bodata_2 ),
	.bsel_2   ( bsel_2   ),
	.bidata_3 ( bidata_3 ),
	.bsel_3   ( bsel_3   ),

	.bodata_4 ( bodata_4 ),
	.bsel_4   ( bsel_4   ),
	.bidata_5 ( bidata_5 ),
	.bsel_5   ( bsel_5   ),

	.bidata_6 ( bidata_6 ),
	.bsel_6   ( bsel_6   ),
	.bodata_7 ( bodata_7 ),
	.bsel_7   ( bsel_7   ),

	.sidata_0 ( sidata_0 ),
	.sodata_0 ( sodata_0 ),
	.ssel_0   ( ssel_0   ),

	.sidata_1 ( sidata_1 ),
	.sodata_1 ( sodata_1 ),
	.ssel_1   ( ssel_1   ),

	.sidata_2 ( sidata_2 ),
	.sodata_2 ( sodata_2 ),
	.ssel_2   ( ssel_2   ),

	.sidata_3 ( sidata_3 ),
	.sodata_3 ( sodata_3 ),
	.ssel_3   ( ssel_3   ),

	.clk      ( clk      ),
	.rst_     ( rst_     )
);

/*
 * Wireless 0 (Bubble, downstream)
 */
txrx rx0 (
	.idata          ( /* NC */          ),
	.odata          ( bidata_0          ),
	.wireless_idata ( wireless_bidata_0 ),
	.wireless_odata ( /* NC */          ),
	.sel            ( bsel_0            ),
	.clk	        ( clk               ),
	.rst_	        ( rst_              )
);
txrx tx1 (
	.idata          ( {bzeros,bodata_1} ),
	.odata          ( /* NC */          ),
	.wireless_idata ( /* NC */          ),
	.wireless_odata ( wireless_bodata_1 ),
	.sel            ( bsel_1            ),
	.clk	        ( clk               ),
	.rst_	        ( rst_              )
);

txrx tx2 (
	.idata          ( bodata_2          ),
	.odata          ( /* NC */          ),
	.wireless_idata ( /* NC */          ),
	.wireless_odata ( wireless_bodata_2 ),
	.sel            ( bsel_2            ),
	.clk	        ( clk               ),
	.rst_	        ( rst_              )
);
txrx rx3 (
	.idata          ( /* NC */          ),
	.odata          ( {null0,bidata_3}  ),
	.wireless_idata ( wireless_bidata_3 ),
	.wireless_odata ( /* NC */          ),
	.sel            ( bsel_3            ),
	.clk	        ( clk               ),
	.rst_	        ( rst_              )
);

/*
 * Wireless 1 (Bubble, upstream)
 */
txrx tx4 (
	.idata          ( bodata_4          ),
	.odata          ( /* NC */          ),
	.wireless_idata ( /* NC */          ),
	.wireless_odata ( wireless_bodata_4 ),
	.sel            ( bsel_4            ),
	.clk	        ( clk               ),
	.rst_	        ( rst_              )
);
txrx rx5 (
	.idata          ( /* NC */          ),
	.odata          ( {null1,bidata_5}  ),
	.wireless_idata ( wireless_bidata_5 ),
	.wireless_odata ( /* NC */          ),
	.sel            ( bsel_5            ),
	.clk	        ( clk               ),
	.rst_	        ( rst_              )
);

txrx rx6 (
	.idata          ( /* NC */          ),
	.odata          ( bidata_6          ),
	.wireless_idata ( wireless_bidata_6 ),
	.wireless_odata ( /* NC */          ),
	.sel            ( bsel_6            ),
	.clk	        ( clk               ),
	.rst_	        ( rst_              )
);
txrx tx7 (
	.idata          ( {bzeros,bodata_7} ),
	.odata          ( /* NC */          ),
	.wireless_idata ( /* NC */          ),
	.wireless_odata ( wireless_bodata_7 ),
	.sel            ( bsel_7            ),
	.clk	        ( clk               ),
	.rst_	        ( rst_              )
);

/*
 * Wireless 3 (Sbus)
 */
txrx txrx0 (
	.idata          ( sodata_0          ),
	.odata          ( sidata_0          ),
	.wireless_idata ( wireless_sidata_0 ),
	.wireless_odata ( wireless_sodata_0 ),
	.sel            ( ssel_0            ),
	.clk	        ( clk               ),
	.rst_	        ( rst_              )
);
txrx txrx1 (
	.idata          ( sodata_1          ),
	.odata          ( sidata_1          ),
	.wireless_idata ( wireless_sidata_1 ),
	.wireless_odata ( wireless_sodata_1 ),
	.sel            ( ssel_1            ),
	.clk	        ( clk               ),
	.rst_	        ( rst_              )
);
txrx txrx2 (
	.idata          ( sodata_2          ),
	.odata          ( sidata_2          ),
	.wireless_idata ( wireless_sidata_2 ),
	.wireless_odata ( wireless_sodata_2 ),
	.sel            ( ssel_2            ),
	.clk	        ( clk               ),
	.rst_	        ( rst_              )
);
txrx txrx3 (
	.idata          ( sodata_3          ),
	.odata          ( sidata_3          ),
	.wireless_idata ( wireless_sidata_3 ),
	.wireless_odata ( wireless_sodata_3 ),
	.sel            ( ssel_3            ),
	.clk	        ( clk               ),
	.rst_	        ( rst_              )
);
/*
 * Chip 0: txrx0
 * Chip 1: txrx1
 * Chip 2: txrx2
 * Chip 3: txrx3
 */
assign	wireless_sidata_0 = (CHIPID == 0) ? wireless_sidata : `TRDATAW_P1'h0;
assign	wireless_sidata_1 = (CHIPID == 1) ? wireless_sidata : `TRDATAW_P1'h0;
assign	wireless_sidata_2 = (CHIPID == 2) ? wireless_sidata : `TRDATAW_P1'h0;
assign	wireless_sidata_3 = (CHIPID == 3) ? wireless_sidata : `TRDATAW_P1'h0;

assign	wireless_sodata   = (CHIPID == 0) ? wireless_sodata_0 :
                            (CHIPID == 1) ? wireless_sodata_1 :
                            (CHIPID == 2) ? wireless_sodata_2 :
                            (CHIPID == 3) ? wireless_sodata_3 : `TRDATAW_P1'hz;

endmodule
