/* $Id: test.v 121 2010-08-15 06:58:37Z matutani $ */
`include "../design/define.h"
`timescale 1ns/100ps
module test;

parameter	STEP = 5.0;	/* 200MHz */

reg			clk;
reg			rst_;
reg			mode;	/* 0: Point-to-Point, 1: Sbus */
reg			burst;	/* 0: solo injection, 1: burst injection */
reg			rdy;	/* Ready for packet injection */
reg	[3:0]		done;
integer			count;	/* Clock counter */
integer			seed;	/* Random seed */

/*
 * Packet counter for each core
 */
reg	[`PKTCNTW:0]	send_0;
reg	[`PKTCNTW:0]	send_1;
reg	[`PKTCNTW:0]	send_2;
reg	[`PKTCNTW:0]	send_3;
reg	[`PKTCNTW:0]	send_4;
reg	[`PKTCNTW:0]	send_5;
reg	[`PKTCNTW:0]	send_6;
reg	[`PKTCNTW:0]	send_7;

reg	[`PKTCNTW:0]	recv_0;
reg	[`PKTCNTW:0]	recv_1;
reg	[`PKTCNTW:0]	recv_2;
reg	[`PKTCNTW:0]	recv_3;
reg	[`PKTCNTW:0]	recv_4;
reg	[`PKTCNTW:0]	recv_5;
reg	[`PKTCNTW:0]	recv_6;
reg	[`PKTCNTW:0]	recv_7;

reg [`EROM_CLK_MSB:0]	rom [0:`EROM_INDEX];	/* Event ROM (event.hex) */

/*
 * Chip 0
 */
reg	[3:0]		idata_0;
wire	[2:0]		odata_0;
reg	[1:0]		sel_0;
reg			wr_0;

/*
 * Chip 1
 */
reg	[3:0]		idata_1;
wire	[2:0]		odata_1;
reg	[1:0]		sel_1;
reg			wr_1;

/*
 * Chip 2
 */
reg	[3:0]		idata_2;
wire	[2:0]		odata_2;
reg	[1:0]		sel_2;
reg			wr_2;

/*
 * Chip 3
 */
reg	[3:0]		idata_3;
wire	[2:0]		odata_3;
reg	[1:0]		sel_3;
reg			wr_3;

/*
 * Wireless
 */
/* Router 6 --> Router 4 (Bubble, downstream) */
wire	[`TRDATAW:0]	wireless_bodata_6;
wire	[`TRDATAW:0]	wireless_bidata_6;
/* Router 4 --> Router 2 (Bubble, downstream) */
wire	[`TRDATAW:0]	wireless_bodata_4;
wire	[`TRDATAW:0]	wireless_bidata_4;
/* Router 2 --> Router 0 (Bubble, downstream) */
wire	[`TRDATAW:0]	wireless_bodata_2;
wire	[`TRDATAW:0]	wireless_bidata_2;

/* Router 1 --> Router 3 (Bubble, upstream) */
wire	[`TRDATAW:0]	wireless_bodata_1;
wire	[`TRDATAW:0]	wireless_bidata_1;
/* Router 3 --> Router 5 (Bubble, upstream) */
wire	[`TRDATAW:0]	wireless_bodata_3;
wire	[`TRDATAW:0]	wireless_bidata_3;
/* Router 5 --> Router 7 (Bubble, upstream) */
wire	[`TRDATAW:0]	wireless_bodata_5;
wire	[`TRDATAW:0]	wireless_bidata_5;

/* Sbus */
wire	[`TRDATAW:0]	wireless_sdata;

/*
 * Chip 0
 */
cube_top #( 0 ) p0 (
	.idata ( idata_0  ),
	.odata ( odata_0  ),
	.sel   ( sel_0    ),
	.wr    ( wr_0     ),

	.probe ( /* NC */ ),

	.wireless_bidata_0	( wireless_bodata_2 ),
	.wireless_bodata_1	( wireless_bidata_2 ),

	.wireless_bodata_2	( /* NC */          ),
	.wireless_bidata_3	( `TRDATAW_P1'b0    ),

	.wireless_bodata_4	( wireless_bodata_1 ),
	.wireless_bidata_5	( wireless_bidata_1 ),

	.wireless_bidata_6	( `TRDATAW_P1'b0    ),
	.wireless_bodata_7	( /* NC */          ),

	.wireless_sidata	( wireless_sdata    ),
	.wireless_sodata	( wireless_sdata    ),

	.clk   ( clk      ),
	.rst_  ( rst_     )
);

/*
 * Chip 1
 */
cube_top #( 1 ) p1 (
	.idata ( idata_1  ),
	.odata ( odata_1  ),
	.sel   ( sel_1    ),
	.wr    ( wr_1     ),

	.probe ( /* NC */ ),

	.wireless_bidata_0	( wireless_bodata_4 ),
	.wireless_bodata_1	( wireless_bidata_4 ),

	.wireless_bodata_2	( wireless_bodata_2 ),
	.wireless_bidata_3	( wireless_bidata_2 ),

	.wireless_bodata_4	( wireless_bodata_3 ),
	.wireless_bidata_5	( wireless_bidata_3 ),

	.wireless_bidata_6	( wireless_bodata_1 ),
	.wireless_bodata_7	( wireless_bidata_1 ),

	.wireless_sidata	( wireless_sdata    ),
	.wireless_sodata	( wireless_sdata    ),

	.clk   ( clk      ),
	.rst_  ( rst_     )
);

/*
 * Chip 2
 */
cube_top #( 2 ) p2 (
	.idata ( idata_2  ),
	.odata ( odata_2  ),
	.sel   ( sel_2    ),
	.wr    ( wr_2     ),

	.probe ( /* NC */ ),

	.wireless_bidata_0	( wireless_bodata_6 ),
	.wireless_bodata_1	( wireless_bidata_6 ),

	.wireless_bodata_2	( wireless_bodata_4 ),
	.wireless_bidata_3	( wireless_bidata_4 ),

	.wireless_bodata_4	( wireless_bodata_5 ),
	.wireless_bidata_5	( wireless_bidata_5 ),

	.wireless_bidata_6	( wireless_bodata_3 ),
	.wireless_bodata_7	( wireless_bidata_3 ),

	.wireless_sidata	( wireless_sdata    ),
	.wireless_sodata	( wireless_sdata    ),

	.clk   ( clk      ),
	.rst_  ( rst_     )
);

/*
 * Chip 3
 */
cube_top #( 3 ) p3 (
	.idata ( idata_3  ),
	.odata ( odata_3  ),
	.sel   ( sel_3    ),
	.wr    ( wr_3     ),

	.probe ( /* NC */ ),

	.wireless_bidata_0	( `TRDATAW_P1'b0    ),
	.wireless_bodata_1	( /* NC */          ),

	.wireless_bodata_2	( wireless_bodata_6 ),
	.wireless_bidata_3	( wireless_bidata_6 ),

	.wireless_bodata_4	( /* NC */          ),
	.wireless_bidata_5	( `TRDATAW_P1'b0    ),

	.wireless_bidata_6	( wireless_bodata_5 ),
	.wireless_bodata_7	( wireless_bidata_5 ),

	.wireless_sidata	( wireless_sdata    ),
	.wireless_sodata	( wireless_sdata    ),

	.clk   ( clk      ),
	.rst_  ( rst_     )
);

/*
 * Clock
 */
always #(STEP / 2) begin
	clk 	<= ~clk;
end
always @(posedge clk) begin
	count	<= count + 1;
end

/*
 * Simulation
 */
initial begin
	#0
	clk	<= 1'b0;
	mode	<= `MODE_P2P;
	burst	<= `Disable;
	rst_	<= `Enable_;
	rdy	<= `Disable;
	count	<= 0;
	done	<= 0;
	idata_0	<= 0;
	idata_1	<= 0;
	idata_2	<= 0;
	idata_3	<= 0;
	sel_0	<= 0;
	sel_1	<= 0;
	sel_2	<= 0;
	sel_3	<= 0;
	wr_0	<= `Disable;
	wr_1	<= `Disable;
	wr_2	<= `Disable;
	wr_3	<= `Disable;
	send_0	<= 0;
	send_1	<= 0;
	send_2	<= 0;
	send_3	<= 0;
	send_4	<= 0;
	send_5	<= 0;
	send_6	<= 0;
	send_7	<= 0;
	recv_0	<= 0;
	recv_1	<= 0;
	recv_2	<= 0;
	recv_3	<= 0;
	recv_4	<= 0;
	recv_5	<= 0;
	recv_6	<= 0;
	recv_7	<= 0;
	$readmemh("event.hex", rom);

	/* Reset */
	#(STEP)
	rst_	<= `Disable_;
	#(STEP / 4)

	/* Initialize Chip ID */ 
	#(STEP)
	sel_0		<= 3;
	wr_0		<= `Enable;
	idata_0[2:0]	<= 0;		/* Chip ID */
	idata_0[3]	<= `Disable;	/* Is top? */
	sel_1		<= 3;
	wr_1		<= `Enable;
	idata_1[2:0]	<= 1;		/* Chip ID */
	idata_1[3]	<= `Disable;	/* Is top? */
	sel_2		<= 3;
	wr_2		<= `Enable;
	idata_2[2:0]	<= 2;		/* Chip ID */
	idata_2[3]	<= `Disable;	/* Is top? */
	sel_3		<= 3;
	wr_3		<= `Enable;
	idata_3[2:0]	<= 3;		/* Chip ID */
	idata_3[3]	<= `Enable;	/* Is top? */
	#(STEP)
	wr_0		<= `Disable;
	wr_1		<= `Disable;
	wr_2		<= `Disable;
	wr_3		<= `Disable;

	/* Initialize NoC mode */ 
	#(STEP)
	if (rom[0] == 0) begin
		mode	<= `MODE_P2P;
		burst	<= `Disable;
	end else if (rom[0] == 1) begin
		mode	<= `MODE_SBUS;
		burst	<= `Disable;
	end else if (rom[0] == 2) begin
		mode	<= `MODE_P2P;
		burst	<= `Enable;
	end else if (rom[0] == 3) begin
		mode	<= `MODE_SBUS;
		burst	<= `Enable;
	end
	#(STEP)
	sel_0		<= 2;
	wr_0		<= `Enable;
	idata_0[1:0]	<= 0;
	idata_0[2]	<= mode;	/* Point-to-point or sbus */
	idata_0[3]	<= 0;
	sel_1		<= 2;
	wr_1		<= `Enable;
	idata_1[1:0]	<= 0;
	idata_1[2]	<= mode;	/* Point-to-point or sbus */
	idata_1[3]	<= 0;
	sel_2		<= 2;
	wr_2		<= `Enable;
	idata_2[1:0]	<= 0;
	idata_2[2]	<= mode;	/* Point-to-point or sbus */
	idata_2[3]	<= 0;
	sel_3		<= 2;
	wr_3		<= `Enable;
	idata_3[1:0]	<= 0;
	idata_3[2]	<= mode;	/* Point-to-point or sbus */
	idata_3[3]	<= 0;
	#(STEP)
	wr_0		<= `Disable;
	wr_1		<= `Disable;
	wr_2		<= `Disable;
	wr_3		<= `Disable;

	/* Start simulation */
	#(STEP)
	rdy	<= `Enable;

	if (burst == `Disable) begin
		/* Wait for all done */
		while (~&done) begin
			#(STEP);
		end
	end else
	if (burst == `Enable) begin
		/* Wait for a while */
		#(STEP * 10000);

		/* Burst mode off */
		#(STEP)
		sel_0		<= 0;
		wr_0		<= `Enable;
		idata_0[0]	<= 0;		/* Core 0 is selected */
		idata_0[3:1]	<= 0;
		sel_1		<= 0;
		wr_1		<= `Enable;
		idata_1[0]	<= 0;		/* Core 2 is selected */
		idata_1[3:1]	<= 0;
		sel_2		<= 0;
		wr_2		<= `Enable;
		idata_2[0]	<= 0;		/* Core 4 is selected */
		idata_2[3:1]	<= 0;
		sel_3		<= 0;
		wr_3		<= `Enable;
		idata_3[0]	<= 0;		/* Core 4 is selected */
		idata_3[3:1]	<= 0;
		#(STEP)
		sel_0		<= 2;
		wr_0		<= `Enable;
		idata_0[1]	<= `Disable;	/* Burst mode off */
		idata_0[2]	<= mode;	/* NoC mode */
		sel_1		<= 2;
		wr_1		<= `Enable;
		idata_1[1]	<= `Disable;	/* Burst mode off */
		idata_1[2]	<= mode;	/* NoC mode */
		sel_2		<= 2;
		wr_2		<= `Enable;
		idata_2[1]	<= `Disable;	/* Burst mode off */
		idata_2[2]	<= mode;	/* NoC mode */
		sel_3		<= 2;
		wr_3		<= `Enable;
		idata_3[1]	<= `Disable;	/* Burst mode off */
		idata_3[2]	<= mode;	/* NoC mode */
		#(STEP)
		sel_0		<= 0;
		wr_0		<= `Enable;
		idata_0[0]	<= 0;		/* Core 0 is selected */
		idata_0[3:1]	<= 0;		/* Injection off */
		sel_1		<= 0;
		wr_1		<= `Enable;
		idata_1[0]	<= 0;		/* Core 2 is selected */
		idata_1[3:1]	<= 0;		/* Injection off */
		sel_2		<= 0;
		wr_2		<= `Enable;
		idata_2[0]	<= 0;		/* Core 4 is selected */
		idata_2[3:1]	<= 0;		/* Injection off */
		sel_3		<= 0;
		wr_3		<= `Enable;
		idata_3[0]	<= 0;		/* Core 4 is selected */
		idata_3[3:1]	<= 0;		/* Injection off */
		#(STEP)
		sel_0		<= 0;
		wr_0		<= `Enable;
		idata_0[0]	<= 1;		/* Core 1 is selected */
		idata_0[3:1]	<= 0;
		sel_1		<= 0;
		wr_1		<= `Enable;
		idata_1[0]	<= 1;		/* Core 3 is selected */
		idata_1[3:1]	<= 0;
		sel_2		<= 0;
		wr_2		<= `Enable;
		idata_2[0]	<= 1;		/* Core 5 is selected */
		idata_2[3:1]	<= 0;
		sel_3		<= 0;
		wr_3		<= `Enable;
		idata_3[0]	<= 1;		/* Core 7 is selected */
		idata_3[3:1]	<= 0;
		#(STEP)
		sel_0		<= 2;
		wr_0		<= `Enable;
		idata_0[1]	<= `Disable;	/* Burst mode off */
		idata_0[2]	<= mode;	/* NoC mode */
		sel_1		<= 2;
		wr_1		<= `Enable;
		idata_1[1]	<= `Disable;	/* Burst mode off */
		idata_1[2]	<= mode;	/* NoC mode */
		sel_2		<= 2;
		wr_2		<= `Enable;
		idata_2[1]	<= `Disable;	/* Burst mode off */
		idata_2[2]	<= mode;	/* NoC mode */
		sel_3		<= 2;
		wr_3		<= `Enable;
		idata_3[1]	<= `Disable;	/* Burst mode off */
		idata_3[2]	<= mode;	/* NoC mode */
		#(STEP)
		sel_0		<= 0;
		wr_0		<= `Enable;
		idata_0[0]	<= 1;		/* Core 1 is selected */
		idata_0[3:1]	<= 0;		/* Injection off */
		sel_1		<= 0;
		wr_1		<= `Enable;
		idata_1[0]	<= 1;		/* Core 3 is selected */
		idata_1[3:1]	<= 0;		/* Injection off */
		sel_2		<= 0;
		wr_2		<= `Enable;
		idata_2[0]	<= 1;		/* Core 5 is selected */
		idata_2[3:1]	<= 0;		/* Injection off */
		sel_3		<= 0;
		wr_3		<= `Enable;
		idata_3[0]	<= 1;		/* Core 7 is selected */
		idata_3[3:1]	<= 0;		/* Injection off */
		#(STEP)
		wr_0		<= `Disable;
		wr_1		<= `Disable;
		wr_2		<= `Disable;
		wr_3		<= `Disable;
	end

	/* Wait for a while */
	#(STEP * 1000);

	/* Print out the number of flits received in each core */
	#(STEP)
	sel_0		<= 0;
	wr_0		<= `Enable;
	idata_0[0]	<= 0;	/* Core 0 is selected */
	idata_0[3:1]	<= 0;
	sel_1		<= 0;
	wr_1		<= `Enable;
	idata_1[0]	<= 0;	/* Core 2 is selected */
	idata_1[3:1]	<= 0;
	sel_2		<= 0;
	wr_2		<= `Enable;
	idata_2[0]	<= 0;	/* Core 4 is selected */
	idata_2[3:1]	<= 0;
	sel_3		<= 0;
	wr_3		<= `Enable;
	idata_3[0]	<= 0;	/* Core 6 is selected */
	idata_3[3:1]	<= 0;
	#(STEP)
	sel_0		<= 2;
	wr_0		<= `Enable;
	idata_0[1:0]	<= 0;
	idata_0[2]	<= mode;	/* NoC mode */
	idata_0[3]	<= `Enable;	/* Core 0 counter read enable */
	sel_1		<= 2;
	wr_1		<= `Enable;
	idata_1[1:0]	<= 0;
	idata_1[2]	<= mode;	/* NoC mode */
	idata_1[3]	<= `Enable;	/* Core 2 counter read enable */
	sel_2		<= 2;
	wr_2		<= `Enable;
	idata_2[1:0]	<= 0;
	idata_2[2]	<= mode;	/* NoC mode */
	idata_2[3]	<= `Enable;	/* Core 4 counter read enable */
	sel_3		<= 2;
	wr_3		<= `Enable;
	idata_3[1:0]	<= 0;
	idata_3[2]	<= mode;	/* NoC mode */
	idata_3[3]	<= `Enable;	/* Core 6 counter read enable */
	#(STEP)
	recv_0[2:0]	<= odata_0;
	recv_2[2:0]	<= odata_1;
	recv_4[2:0]	<= odata_2;
	recv_6[2:0]	<= odata_3;
	#(STEP)
	recv_0[5:3]	<= odata_0;
	recv_2[5:3]	<= odata_1;
	recv_4[5:3]	<= odata_2;
	recv_6[5:3]	<= odata_3;
	#(STEP)
	recv_0[8:6]	<= odata_0;
	recv_2[8:6]	<= odata_1;
	recv_4[8:6]	<= odata_2;
	recv_6[8:6]	<= odata_3;
	#(STEP)
	recv_0[11:9]	<= odata_0;
	recv_2[11:9]	<= odata_1;
	recv_4[11:9]	<= odata_2;
	recv_6[11:9]	<= odata_3;
	#(STEP)
	recv_0[14:12]	<= odata_0;
	recv_2[14:12]	<= odata_1;
	recv_4[14:12]	<= odata_2;
	recv_6[14:12]	<= odata_3;
	#(STEP)
	recv_0[17:15]	<= odata_0;
	recv_2[17:15]	<= odata_1;
	recv_4[17:15]	<= odata_2;
	recv_6[17:15]	<= odata_3;
	#(STEP)
	recv_0[20:18]	<= odata_0;
	recv_2[20:18]	<= odata_1;
	recv_4[20:18]	<= odata_2;
	recv_6[20:18]	<= odata_3;
	#(STEP)
	recv_0[23:21]	<= odata_0;
	recv_2[23:21]	<= odata_1;
	recv_4[23:21]	<= odata_2;
	recv_6[23:21]	<= odata_3;
	#(STEP)
	recv_0[26:24]	<= odata_0;
	recv_2[26:24]	<= odata_1;
	recv_4[26:24]	<= odata_2;
	recv_6[26:24]	<= odata_3;
	#(STEP)
	recv_0[29:27]	<= odata_0;
	recv_2[29:27]	<= odata_1;
	recv_4[29:27]	<= odata_2;
	recv_6[29:27]	<= odata_3;
	#(STEP)
	recv_0[32:30]	<= odata_0;
	recv_2[32:30]	<= odata_1;
	recv_4[32:30]	<= odata_2;
	recv_6[32:30]	<= odata_3;
	#(STEP)
	recv_0[35:33]	<= odata_0;
	recv_2[35:33]	<= odata_1;
	recv_4[35:33]	<= odata_2;
	recv_6[35:33]	<= odata_3;
	#(STEP)
	recv_0[38:36]	<= odata_0;
	recv_2[38:36]	<= odata_1;
	recv_4[38:36]	<= odata_2;
	recv_6[38:36]	<= odata_3;
	#(STEP)
	recv_0[41:39]	<= odata_0;
	recv_2[41:39]	<= odata_1;
	recv_4[41:39]	<= odata_2;
	recv_6[41:39]	<= odata_3;
	#(STEP)
	recv_0[44:42]	<= odata_0;
	recv_2[44:42]	<= odata_1;
	recv_4[44:42]	<= odata_2;
	recv_6[44:42]	<= odata_3;
	wr_0		<= `Disable;
	wr_1		<= `Disable;
	wr_2		<= `Disable;
	wr_3		<= `Disable;

	#(STEP)
	sel_0		<= 0;
	wr_0		<= `Enable;
	idata_0[0]	<= 1;	/* Core 1 is selected */
	idata_0[3:1]	<= 0;
	sel_1		<= 0;
	wr_1		<= `Enable;
	idata_1[0]	<= 1;	/* Core 3 is selected */
	idata_1[3:1]	<= 0;
	sel_2		<= 0;
	wr_2		<= `Enable;
	idata_2[0]	<= 1;	/* Core 5 is selected */
	idata_2[3:1]	<= 0;
	sel_3		<= 0;
	wr_3		<= `Enable;
	idata_3[0]	<= 1;	/* Core 7 is selected */
	idata_3[3:1]	<= 0;
	#(STEP)
	sel_0		<= 2;
	wr_0		<= `Enable;
	idata_0[1:0]	<= 0;
	idata_0[2]	<= mode;	/* NoC mode */
	idata_0[3]	<= `Enable;	/* Core 1 counter read enable */
	sel_1		<= 2;
	wr_1		<= `Enable;
	idata_1[1:0]	<= 0;
	idata_1[2]	<= mode;	/* NoC mode */
	idata_1[3]	<= `Enable;	/* Core 3 counter read enable */
	sel_2		<= 2;
	wr_2		<= `Enable;
	idata_2[1:0]	<= 0;
	idata_2[2]	<= mode;	/* NoC mode */
	idata_2[3]	<= `Enable;	/* Core 5 counter read enable */
	sel_3		<= 2;
	wr_3		<= `Enable;
	idata_3[1:0]	<= 0;
	idata_3[2]	<= mode;	/* NoC mode */
	idata_3[3]	<= `Enable;	/* Core 7 counter read enable */
	#(STEP)
	recv_1[2:0]	<= odata_0;
	recv_3[2:0]	<= odata_1;
	recv_5[2:0]	<= odata_2;
	recv_7[2:0]	<= odata_3;
	#(STEP)
	recv_1[5:3]	<= odata_0;
	recv_3[5:3]	<= odata_1;
	recv_5[5:3]	<= odata_2;
	recv_7[5:3]	<= odata_3;
	#(STEP)
	recv_1[8:6]	<= odata_0;
	recv_3[8:6]	<= odata_1;
	recv_5[8:6]	<= odata_2;
	recv_7[8:6]	<= odata_3;
	#(STEP)
	recv_1[11:9]	<= odata_0;
	recv_3[11:9]	<= odata_1;
	recv_5[11:9]	<= odata_2;
	recv_7[11:9]	<= odata_3;
	#(STEP)
	recv_1[14:12]	<= odata_0;
	recv_3[14:12]	<= odata_1;
	recv_5[14:12]	<= odata_2;
	recv_7[14:12]	<= odata_3;
	#(STEP)
	recv_1[17:15]	<= odata_0;
	recv_3[17:15]	<= odata_1;
	recv_5[17:15]	<= odata_2;
	recv_7[17:15]	<= odata_3;
	#(STEP)
	recv_1[20:18]	<= odata_0;
	recv_3[20:18]	<= odata_1;
	recv_5[20:18]	<= odata_2;
	recv_7[20:18]	<= odata_3;
	#(STEP)
	recv_1[23:21]	<= odata_0;
	recv_3[23:21]	<= odata_1;
	recv_5[23:21]	<= odata_2;
	recv_7[23:21]	<= odata_3;
	#(STEP)
	recv_1[26:24]	<= odata_0;
	recv_3[26:24]	<= odata_1;
	recv_5[26:24]	<= odata_2;
	recv_7[26:24]	<= odata_3;
	#(STEP)
	recv_1[29:27]	<= odata_0;
	recv_3[29:27]	<= odata_1;
	recv_5[29:27]	<= odata_2;
	recv_7[29:27]	<= odata_3;
	#(STEP)
	recv_1[32:30]	<= odata_0;
	recv_3[32:30]	<= odata_1;
	recv_5[32:30]	<= odata_2;
	recv_7[32:30]	<= odata_3;
	#(STEP)
	recv_1[35:33]	<= odata_0;
	recv_3[35:33]	<= odata_1;
	recv_5[35:33]	<= odata_2;
	recv_7[35:33]	<= odata_3;
	#(STEP)
	recv_1[38:36]	<= odata_0;
	recv_3[38:36]	<= odata_1;
	recv_5[38:36]	<= odata_2;
	recv_7[38:36]	<= odata_3;
	#(STEP)
	recv_1[41:39]	<= odata_0;
	recv_3[41:39]	<= odata_1;
	recv_5[41:39]	<= odata_2;
	recv_7[41:39]	<= odata_3;
	#(STEP)
	recv_1[44:42]	<= odata_0;
	recv_3[44:42]	<= odata_1;
	recv_5[44:42]	<= odata_2;
	recv_7[44:42]	<= odata_3;
	wr_0		<= `Disable;
	wr_1		<= `Disable;
	wr_2		<= `Disable;
	wr_3		<= `Disable;

	#(STEP)
	$write("Core 0 received %d flits \n", recv_0);
	$write("Core 1 received %d flits \n", recv_1);
	$write("Core 2 received %d flits \n", recv_2);
	$write("Core 3 received %d flits \n", recv_3);
	$write("Core 4 received %d flits \n", recv_4);
	$write("Core 5 received %d flits \n", recv_5);
	$write("Core 6 received %d flits \n", recv_6);
	$write("Core 7 received %d flits \n", recv_7);
	$write("*** Total  sent     %d flits \n", send_0 + send_1 + send_2 + send_3 + send_4 + send_5 + send_6 + send_7);
	$write("*** Total  received %d flits \n", recv_0 + recv_1 + recv_2 + recv_3 + recv_4 + recv_5 + recv_6 + recv_7);
	$finish;
end

/*
 * Packet injector (Chip 0)
 */
reg	[`EROM_INDEXW:0]	ptr_0;
reg	[`EROM_INDEXW:0]	num_0;
reg	[`EROM_CLK_MSB:0] 	event_0;
reg	[`NODEW:0]		dest_0;
reg	[`VCHW:0]		vch_0;
reg				ordy_0;
initial begin
	#(STEP / 4)
	/* Wait for ready */
	while (~rdy) begin
		#(STEP);
	end
	num_0	<= rom[1];	/* Number of events in the event ROM */
	event_0	<= rom[2];	/* The first event */
	ptr_0   <= 2;		/* Event ROM pointer */
	#(STEP)
	while (ptr_0 < num_0 + 2) begin
		/*
		 * Core 0
		 */
		if (event_0[`EROM_SRC_MSB:`EROM_SRC_LSB] == 0) begin
			sel_0		<= 0;
			wr_0		<= `Enable;
			idata_0[0]	<= 0;	/* Core 0 is selected */
			idata_0[3:1]	<= 0;
			dest_0		<= event_0[`EROM_DST_MSB:`EROM_DST_LSB];
			vch_0		<= event_0[`EROM_VCH_MSB:`EROM_VCH_LSB];
			#(STEP)
			wr_0		<= `Disable;
			if (vch_0 == 0)
				ordy_0	<= odata_0[0];
			if (vch_0 == 1)
				ordy_0	<= odata_0[1];
			#(STEP)
			if (ordy_0 && event_0[`EROM_CLK_MSB:`EROM_CLK_LSB] <= count) begin
				sel_0		<= 1;
				wr_0		<= `Enable;
				idata_0[3:0]	<= dest_0;	/* Dest addr is set */
				#(STEP)
				sel_0		<= 2;
				wr_0		<= `Enable;
				idata_0[0]	<= vch_0;	/* VC ID is set */
				idata_0[1]	<= burst;	/* Burst is off */
				idata_0[2]	<= mode;	/* Point-to-point or sbus */
				idata_0[3]	<= 0;
				#(STEP)
				sel_0		<= 0;
				wr_0		<= `Enable;
				idata_0[0]	<= 0;		/* Core 0 is selected */
				idata_0[1]	<= `Enable;	/* Injection! */
				idata_0[3:2]	<= 0;		/* To avoid dummy state */
				send_0		<= send_0 + `PKTLEN_P1;
				#(STEP)
				$write("%10d >>> Core %3d sends ( src: %3d dst: %3d vc: %d ) \n", count, 0, 0, dest_0, vch_0);
				wr_0		<= `Disable;
				ptr_0		<= ptr_0 + 1;
				event_0		<= rom[ptr_0 + 1];
			end
			#(STEP);
		end
		/*
		 * Core 1
		 */
		else if (event_0[`EROM_SRC_MSB:`EROM_SRC_LSB] == 1) begin
			sel_0		<= 0;
			wr_0		<= `Enable;
			idata_0[0]	<= 1;	/* Core 1 is selected */
			idata_0[3:1]	<= 0;
			dest_0		<= event_0[`EROM_DST_MSB:`EROM_DST_LSB];
			vch_0		<= event_0[`EROM_VCH_MSB:`EROM_VCH_LSB];
			#(STEP)
			wr_0		<= `Disable;
			if (vch_0 == 0)
				ordy_0	<= odata_0[0];
			if (vch_0 == 1)
				ordy_0	<= odata_0[1];
			#(STEP)
			if (ordy_0 && event_0[`EROM_CLK_MSB:`EROM_CLK_LSB] <= count) begin
				sel_0		<= 1;
				wr_0		<= `Enable;
				idata_0[3:0]	<= dest_0;	/* Dest addr is set */
				#(STEP)
				sel_0		<= 2;
				wr_0		<= `Enable;
				idata_0[0]	<= vch_0;	/* VC ID is set */
				idata_0[1]	<= burst;	/* Burst is off */
				idata_0[2]	<= mode;	/* Point-to-point or sbus */
				idata_0[3]	<= 0;
				#(STEP)
				sel_0		<= 0;
				wr_0		<= `Enable;
				idata_0[0]	<= 1;		/* Core 1 is selected */
				idata_0[1]	<= `Enable;	/* Injection! */
				idata_0[3:2]	<= 0;		/* To avoid dummy state */
				send_1		<= send_1 + `PKTLEN_P1;
				#(STEP)
				$write("%10d >>> Core %3d sends ( src: %3d dst: %3d vc: %d ) \n", count, 1, 1, dest_0, vch_0);
				wr_0		<= `Disable;
				ptr_0		<= ptr_0 + 1;
				event_0		<= rom[ptr_0 + 1];
			end
			#(STEP);
		end 
		else begin
			ptr_0		<= ptr_0 + 1;
			event_0		<= rom[ptr_0 + 1];
			#(STEP);
		end
	end
	done[0]	<= `Enable;
end

/*
 * Packet injector (Chip 1)
 */
reg	[`EROM_INDEXW:0]	ptr_1;
reg	[`EROM_INDEXW:0]	num_1;
reg	[`EROM_CLK_MSB:0] 	event_1;
reg	[`NODEW:0]		dest_1;
reg	[`VCHW:0]		vch_1;
reg				ordy_1;
initial begin
	#(STEP / 4)
	/* Wait for ready */
	while (~rdy) begin
		#(STEP);
	end
	num_1	<= rom[1];	/* Number of events in the event ROM */
	event_1	<= rom[2];	/* The first event */
	ptr_1   <= 2;		/* Event ROM pointer */
	#(STEP)
	while (ptr_1 < num_1 + 2) begin
		/*
		 * Core 2
		 */
		if (event_1[`EROM_SRC_MSB:`EROM_SRC_LSB] == 2) begin
			sel_1		<= 0;
			wr_1		<= `Enable;
			idata_1[0]	<= 0;	/* Core 2 is selected */
			idata_1[3:1]	<= 0;
			dest_1		<= event_1[`EROM_DST_MSB:`EROM_DST_LSB];
			vch_1		<= event_1[`EROM_VCH_MSB:`EROM_VCH_LSB];
			#(STEP)
			wr_1		<= `Disable;
			if (vch_1 == 0)
				ordy_1	<= odata_1[0];
			if (vch_1 == 1)
				ordy_1	<= odata_1[1];
			#(STEP)
			if (ordy_1 && event_1[`EROM_CLK_MSB:`EROM_CLK_LSB] <= count) begin
				sel_1		<= 1;
				wr_1		<= `Enable;
				idata_1[3:0]	<= dest_1;	/* Dest addr is set */
				#(STEP)
				sel_1		<= 2;
				wr_1		<= `Enable;
				idata_1[0]	<= vch_1;	/* VC ID is set */
				idata_1[1]	<= burst;	/* Burst is off */
				idata_1[2]	<= mode;	/* Point-to-point or sbus */
				idata_1[3]	<= 0;
				#(STEP)
				sel_1		<= 0;
				wr_1		<= `Enable;
				idata_1[0]	<= 0;		/* Core 2 is selected */
				idata_1[1]	<= `Enable;	/* Injection! */
				idata_1[3:2]	<= 0;		/* To avoid dummy state */
				send_2		<= send_2 + `PKTLEN_P1;
				#(STEP)
				$write("%10d >>> Core %3d sends ( src: %3d dst: %3d vc: %d ) \n", count, 2, 2, dest_1, vch_1);
				wr_1		<= `Disable;
				ptr_1		<= ptr_1 + 1;
				event_1		<= rom[ptr_1 + 1];
			end
			#(STEP);
		end
		/*
		 * Core 3
		 */
		else if (event_1[`EROM_SRC_MSB:`EROM_SRC_LSB] == 3) begin
			sel_1		<= 0;
			wr_1		<= `Enable;
			idata_1[0]	<= 1;	/* Core 3 is selected */
			idata_1[3:1]	<= 0;
			dest_1		<= event_1[`EROM_DST_MSB:`EROM_DST_LSB];
			vch_1		<= event_1[`EROM_VCH_MSB:`EROM_VCH_LSB];
			#(STEP)
			wr_1		<= `Disable;
			if (vch_1 == 0)
				ordy_1	<= odata_1[0];
			if (vch_1 == 1)
				ordy_1	<= odata_1[1];
			#(STEP)
			if (ordy_1 && event_1[`EROM_CLK_MSB:`EROM_CLK_LSB] <= count) begin
				sel_1		<= 1;
				wr_1		<= `Enable;
				idata_1[3:0]	<= dest_1;	/* Dest addr is set */
				#(STEP)
				sel_1		<= 2;
				wr_1		<= `Enable;
				idata_1[0]	<= vch_1;	/* VC ID is set */
				idata_1[1]	<= burst;	/* Burst is off */
				idata_1[2]	<= mode;	/* Point-to-point or sbus */
				idata_1[3]	<= 0;
				#(STEP)
				sel_1		<= 0;
				wr_1		<= `Enable;
				idata_1[0]	<= 1;		/* Core 3 is selected */
				idata_1[1]	<= `Enable;	/* Injection! */
				idata_1[3:2]	<= 0;		/* To avoid dummy state */
				send_3		<= send_3 + `PKTLEN_P1;
				#(STEP)
				$write("%10d >>> Core %3d sends ( src: %3d dst: %3d vc: %d ) \n", count, 3, 3, dest_1, vch_1);
				wr_1		<= `Disable;
				ptr_1		<= ptr_1 + 1;
				event_1		<= rom[ptr_1 + 1];
			end
			#(STEP);
		end 
		else begin
			ptr_1		<= ptr_1 + 1;
			event_1		<= rom[ptr_1 + 1];
			#(STEP);
		end
	end
	done[1]	<= `Enable;
end

/*
 * Packet injector (Chip 2)
 */
reg	[`EROM_INDEXW:0]	ptr_2;
reg	[`EROM_INDEXW:0]	num_2;
reg	[`EROM_CLK_MSB:0] 	event_2;
reg	[`NODEW:0]		dest_2;
reg	[`VCHW:0]		vch_2;
reg				ordy_2;
initial begin
	#(STEP / 4)
	/* Wait for ready */
	while (~rdy) begin
		#(STEP);
	end
	num_2	<= rom[1];	/* Number of events in the event ROM */
	event_2	<= rom[2];	/* The first event */
	ptr_2   <= 2;		/* Event ROM pointer */
	#(STEP)
	while (ptr_2 < num_2 + 2) begin
		/*
		 * Core 4
		 */
		if (event_2[`EROM_SRC_MSB:`EROM_SRC_LSB] == 4) begin
			sel_2		<= 0;
			wr_2		<= `Enable;
			idata_2[0]	<= 0;	/* Core 4 is selected */
			idata_2[3:1]	<= 0;
			dest_2		<= event_2[`EROM_DST_MSB:`EROM_DST_LSB];
			vch_2		<= event_2[`EROM_VCH_MSB:`EROM_VCH_LSB];
			#(STEP)
			wr_2		<= `Disable;
			if (vch_2 == 0)
				ordy_2	<= odata_2[0];
			if (vch_2 == 1)
				ordy_2	<= odata_2[1];
			#(STEP)
			if (ordy_2 && event_2[`EROM_CLK_MSB:`EROM_CLK_LSB] <= count) begin
				sel_2		<= 1;
				wr_2		<= `Enable;
				idata_2[3:0]	<= dest_2;	/* Dest addr is set */
				#(STEP)
				sel_2		<= 2;
				wr_2		<= `Enable;
				idata_2[0]	<= vch_2;	/* VC ID is set */
				idata_2[1]	<= burst;	/* Burst is off */
				idata_2[2]	<= mode;	/* Point-to-point or sbus */
				idata_2[3]	<= 0;
				#(STEP)
				sel_2		<= 0;
				wr_2		<= `Enable;
				idata_2[0]	<= 0;		/* Core 4 is selected */
				idata_2[1]	<= `Enable;	/* Injection! */
				idata_2[3:2]	<= 0;		/* To avoid dummy state */
				send_4		<= send_4 + `PKTLEN_P1;
				#(STEP)
				$write("%10d >>> Core %3d sends ( src: %3d dst: %3d vc: %d ) \n", count, 4, 4, dest_2, vch_2);
				wr_2		<= `Disable;
				ptr_2		<= ptr_2 + 1;
				event_2		<= rom[ptr_2 + 1];
			end
			#(STEP);
		end
		/*
		 * Core 5
		 */
		else if (event_2[`EROM_SRC_MSB:`EROM_SRC_LSB] == 5) begin
			sel_2		<= 0;
			wr_2		<= `Enable;
			idata_2[0]	<= 1;	/* Core 5 is selected */
			idata_2[3:1]	<= 0;
			dest_2		<= event_2[`EROM_DST_MSB:`EROM_DST_LSB];
			vch_2		<= event_2[`EROM_VCH_MSB:`EROM_VCH_LSB];
			#(STEP)
			wr_2		<= `Disable;
			if (vch_2 == 0)
				ordy_2	<= odata_2[0];
			if (vch_2 == 1)
				ordy_2	<= odata_2[1];
			#(STEP)
			if (ordy_2 && event_2[`EROM_CLK_MSB:`EROM_CLK_LSB] <= count) begin
				sel_2		<= 1;
				wr_2		<= `Enable;
				idata_2[3:0]	<= dest_2;	/* Dest addr is set */
				#(STEP)
				sel_2		<= 2;
				wr_2		<= `Enable;
				idata_2[0]	<= vch_2;	/* VC ID is set */
				idata_2[1]	<= burst;	/* Burst is off */
				idata_2[2]	<= mode;	/* Point-to-point or sbus */
				idata_2[3]	<= 0;
				#(STEP)
				sel_2		<= 0;
				wr_2		<= `Enable;
				idata_2[0]	<= 1;		/* Core 5 is selected */
				idata_2[1]	<= `Enable;	/* Injection! */
				idata_2[3:2]	<= 0;		/* To avoid dummy state */
				send_5		<= send_5 + `PKTLEN_P1;
				#(STEP)
				$write("%10d >>> Core %3d sends ( src: %3d dst: %3d vc: %d ) \n", count, 5, 5, dest_2, vch_2);
				wr_2		<= `Disable;
				ptr_2		<= ptr_2 + 1;
				event_2		<= rom[ptr_2 + 1];
			end
			#(STEP);
		end 
		else begin
			ptr_2		<= ptr_2 + 1;
			event_2		<= rom[ptr_2 + 1];
			#(STEP);
		end
	end
	done[2]	<= `Enable;
end

/*
 * Packet injector (Chip 3)
 */
reg	[`EROM_INDEXW:0]	ptr_3;
reg	[`EROM_INDEXW:0]	num_3;
reg	[`EROM_CLK_MSB:0] 	event_3;
reg	[`NODEW:0]		dest_3;
reg	[`VCHW:0]		vch_3;
reg				ordy_3;
initial begin
	#(STEP / 4)
	/* Wait for ready */
	while (~rdy) begin
		#(STEP);
	end
	num_3	<= rom[1];	/* Number of events in the event ROM */
	event_3	<= rom[2];	/* The first event */
	ptr_3   <= 2;		/* Event ROM pointer */
	#(STEP)
	while (ptr_3 < num_3 + 2) begin
		/*
		 * Core 6
		 */
		if (event_3[`EROM_SRC_MSB:`EROM_SRC_LSB] == 6) begin
			sel_3		<= 0;
			wr_3		<= `Enable;
			idata_3[0]	<= 0;	/* Core 6 is selected */
			idata_3[3:1]	<= 0;
			dest_3		<= event_3[`EROM_DST_MSB:`EROM_DST_LSB];
			vch_3		<= event_3[`EROM_VCH_MSB:`EROM_VCH_LSB];
			#(STEP)
			wr_3		<= `Disable;
			if (vch_3 == 0)
				ordy_3	<= odata_3[0];
			if (vch_3 == 1)
				ordy_3	<= odata_3[1];
			#(STEP)
			if (ordy_3 && event_3[`EROM_CLK_MSB:`EROM_CLK_LSB] <= count) begin
				sel_3		<= 1;
				wr_3		<= `Enable;
				idata_3[3:0]	<= dest_3;	/* Dest addr is set */
				#(STEP)
				sel_3		<= 2;
				wr_3		<= `Enable;
				idata_3[0]	<= vch_3;	/* VC ID is set */
				idata_3[1]	<= burst;	/* Burst is off */
				idata_3[2]	<= mode;	/* Point-to-point or sbus */
				idata_3[3]	<= 0;
				#(STEP)
				sel_3		<= 0;
				wr_3		<= `Enable;
				idata_3[0]	<= 0;		/* Core 6 is selected */
				idata_3[1]	<= `Enable;	/* Injection! */
				idata_3[3:2]	<= 0;		/* To avoid dummy state */
				send_6		<= send_6 + `PKTLEN_P1;
				#(STEP)
				$write("%10d >>> Core %3d sends ( src: %3d dst: %3d vc: %d ) \n", count, 6, 6, dest_3, vch_3);
				wr_3		<= `Disable;
				ptr_3		<= ptr_3 + 1;
				event_3		<= rom[ptr_3 + 1];
			end
			#(STEP);
		end
		/*
		 * Core 7
		 */
		else if (event_3[`EROM_SRC_MSB:`EROM_SRC_LSB] == 7) begin
			sel_3		<= 0;
			wr_3		<= `Enable;
			idata_3[0]	<= 1;	/* Core 7 is selected */
			idata_3[3:1]	<= 0;
			dest_3		<= event_3[`EROM_DST_MSB:`EROM_DST_LSB];
			vch_3		<= event_3[`EROM_VCH_MSB:`EROM_VCH_LSB];
			#(STEP)
			wr_3		<= `Disable;
			if (vch_3 == 0)
				ordy_3	<= odata_3[0];
			if (vch_3 == 1)
				ordy_3	<= odata_3[1];
			#(STEP)
			if (ordy_3 && event_3[`EROM_CLK_MSB:`EROM_CLK_LSB] <= count) begin
				sel_3		<= 1;
				wr_3		<= `Enable;
				idata_3[3:0]	<= dest_3;	/* Dest addr is set */
				#(STEP)
				sel_3		<= 2;
				wr_3		<= `Enable;
				idata_3[0]	<= vch_3;	/* VC ID is set */
				idata_3[1]	<= burst;	/* Burst is off */
				idata_3[2]	<= mode;	/* Point-to-point or sbus */
				idata_3[3]	<= 0;
				#(STEP)
				sel_3		<= 0;
				wr_3		<= `Enable;
				idata_3[0]	<= 1;		/* Core 7 is selected */
				idata_3[1]	<= `Enable;	/* Injection! */
				idata_3[3:2]	<= 0;		/* To avoid dummy state */
				send_7		<= send_7 + `PKTLEN_P1;
				#(STEP)
				$write("%10d >>> Core %3d sends ( src: %3d dst: %3d vc: %d ) \n", count, 7, 7, dest_3, vch_3);
				wr_3		<= `Disable;
				ptr_3		<= ptr_3 + 1;
				event_3		<= rom[ptr_3 + 1];
			end
			#(STEP);
		end 
		else begin
			ptr_3		<= ptr_3 + 1;
			event_3		<= rom[ptr_3 + 1];
			#(STEP);
		end
	end
	done[3]	<= `Enable;
end

/*
 * Confirmation tests
 */
always @(posedge clk) begin
	/* Unused TXRX for Sbus @ Chip 0 */
	if (p0.txrx1.sel != `TRSEL_IDLE) begin
		$write("Error: p0.txrx1.sel is not idle.\n");
		$finish;
	end
	if (p0.txrx2.sel != `TRSEL_IDLE) begin
		$write("Error: p0.txrx2.sel is not idle.\n");
		$finish;
	end
	if (p0.txrx3.sel != `TRSEL_IDLE) begin
		$write("Error: p0.txrx3.sel is not idle.\n");
		$finish;
	end
	/* Unused TXRX for Sbus @ Chip 1 */
	if (p1.txrx0.sel != `TRSEL_IDLE) begin
		$write("Error: p1.txrx0.sel is not idle.\n");
		$finish;
	end
	if (p1.txrx2.sel != `TRSEL_IDLE) begin
		$write("Error: p1.txrx2.sel is not idle.\n");
		$finish;
	end
	if (p1.txrx3.sel != `TRSEL_IDLE) begin
		$write("Error: p1.txrx3.sel is not idle.\n");
		$finish;
	end
	/* Unused TXRX for Sbus @ Chip 2 */
	if (p2.txrx0.sel != `TRSEL_IDLE) begin
		$write("Error: p2.txrx0.sel is not idle.\n");
		$finish;
	end
	if (p2.txrx1.sel != `TRSEL_IDLE) begin
		$write("Error: p2.txrx1.sel is not idle.\n");
		$finish;
	end
	if (p2.txrx3.sel != `TRSEL_IDLE) begin
		$write("Error: p2.txrx3.sel is not idle.\n");
		$finish;
	end
	/* Unused TXRX for Sbus @ Chip 3 */
	if (p3.txrx0.sel != `TRSEL_IDLE) begin
		$write("Error: p3.txrx0.sel is not idle.\n");
		$finish;
	end
	if (p3.txrx1.sel != `TRSEL_IDLE) begin
		$write("Error: p3.txrx1.sel is not idle.\n");
		$finish;
	end
	if (p3.txrx2.sel != `TRSEL_IDLE) begin
		$write("Error: p3.txrx2.sel is not idle.\n");
		$finish;
	end

	/* Only a single TX is allowed at any time for Sbus  */
	if (p0.txrx0.sel == `TRSEL_TX) begin
		if ( (p1.txrx1.sel == `TRSEL_TX) ||
		     (p2.txrx2.sel == `TRSEL_TX) ||
		     (p3.txrx3.sel == `TRSEL_TX) ) begin
			$write("Error: TX conflict when p0.txrx0.sel is TX.\n");
			$finish;
		end
	end
	if (p1.txrx1.sel == `TRSEL_TX) begin
		if ( (p0.txrx0.sel == `TRSEL_TX) ||
		     (p2.txrx2.sel == `TRSEL_TX) ||
		     (p3.txrx3.sel == `TRSEL_TX) ) begin
			$write("Error: TX conflict when p1.txrx1.sel is TX.\n");
			$finish;
		end
	end
	if (p2.txrx2.sel == `TRSEL_TX) begin
		if ( (p0.txrx0.sel == `TRSEL_TX) ||
		     (p1.txrx1.sel == `TRSEL_TX) ||
		     (p3.txrx3.sel == `TRSEL_TX) ) begin
			$write("Error: TX conflict when p2.txrx2.sel is TX.\n");
			$finish;
		end
	end
	if (p3.txrx3.sel == `TRSEL_TX) begin
		if ( (p0.txrx0.sel == `TRSEL_TX) ||
		     (p1.txrx1.sel == `TRSEL_TX) ||
		     (p2.txrx2.sel == `TRSEL_TX) ) begin
			$write("Error: TX conflict when p3.txrx3.sel is TX.\n");
			$finish;
		end
	end

	/* Unused TXRX for Point-to-point @ Chip 0 */
	if (p0.tx2.sel != `TRSEL_IDLE) begin
		$write("Error: p0.tx2.sel is not idle.\n");
		$finish;
	end
	if (p0.rx3.sel != `TRSEL_IDLE) begin
		$write("Error: p0.rx3.sel is not idle.\n");
		$finish;
	end
	if (p0.rx6.sel != `TRSEL_IDLE) begin
		$write("Error: p0.rx6.sel is not idle.\n");
		$finish;
	end
	if (p0.tx7.sel != `TRSEL_IDLE) begin
		$write("Error: p0.tx7.sel is not idle.\n");
		$finish;
	end
	/* Unused TXRX for Point-to-point @ Chip 3 */
	if (p3.rx0.sel != `TRSEL_IDLE) begin
		$write("Error: p3.rx0.sel is not idle.\n");
		$finish;
	end
	if (p3.tx1.sel != `TRSEL_IDLE) begin
		$write("Error: p3.tx1.sel is not idle.\n");
		$finish;
	end
	if (p3.tx4.sel != `TRSEL_IDLE) begin
		$write("Error: p3.tx4.sel is not idle.\n");
		$finish;
	end
	if (p3.rx5.sel != `TRSEL_IDLE) begin
		$write("Error: p3.rx5.sel is not idle.\n");
		$finish;
	end

	/* Credit inductors for Point-to-point cannot use 2'b11 (Chip 0). */
	if (p0.tx1.wireless_odata[1:0] == 2'b11) begin
		$write("Error: p0.tx1.wireless_odata is prohibited value.\n");
		$finish;
	end
	if (p0.tx7.wireless_odata[1:0] == 2'b11) begin
		$write("Error: p0.tx7.wireless_odata is prohibited value.\n");
		$finish;
	end
	/* Credit inductors for Point-to-point cannot use 2'b11 (Chip 1). */
	if (p1.tx1.wireless_odata[1:0] == 2'b11) begin
		$write("Error: p1.tx1.wireless_odata is prohibited value.\n");
		$finish;
	end
	if (p1.tx7.wireless_odata[1:0] == 2'b11) begin
		$write("Error: p1.tx7.wireless_odata is prohibited value.\n");
		$finish;
	end
	/* Credit inductors for Point-to-point cannot use 2'b11 (Chip 2). */
	if (p2.tx1.wireless_odata[1:0] == 2'b11) begin
		$write("Error: p2.tx1.wireless_odata is prohibited value.\n");
		$finish;
	end
	if (p2.tx7.wireless_odata[1:0] == 2'b11) begin
		$write("Error: p2.tx7.wireless_odata is prohibited value.\n");
		$finish;
	end
	/* Credit inductors for Point-to-point cannot use 2'b11 (Chip 3). */
	if (p3.tx1.wireless_odata[1:0] == 2'b11) begin
		$write("Error: p3.tx1.wireless_odata is prohibited value.\n");
		$finish;
	end
	if (p3.tx7.wireless_odata[1:0] == 2'b11) begin
		$write("Error: p3.tx7.wireless_odata is prohibited value.\n");
		$finish;
	end

	/* Cube's dummy must be Disabled at any time. */
	if (p0.c0.dummy == `Enable) begin
		$write("Error: p0.c0.dummy is prohibited value.\n");
		$finish;
	end
	if (p1.c0.dummy == `Enable) begin
		$write("Error: p1.c0.dummy is prohibited value.\n");
		$finish;
	end
	if (p2.c0.dummy == `Enable) begin
		$write("Error: p2.c0.dummy is prohibited value.\n");
		$finish;
	end
	if (p3.c0.dummy == `Enable) begin
		$write("Error: p3.c0.dummy is prohibited value.\n");
		$finish;
	end
end

/*
 * Dump file
 */
/*
initial begin
	$dumpfile("test.vcd");
	$dumpvars(0, test);
end
*/

/*
 * SDF annotation
 */
`ifdef __SDF__
initial begin
	$sdf_annotate("cube_pt.sdf", TEST.P0.C0, , , "MAXIMUM");
	$sdf_annotate("cube_pt.sdf", TEST.P1.C0, , , "MAXIMUM");
	$sdf_annotate("cube_pt.sdf", TEST.P2.C0, , , "MAXIMUM");
	$sdf_annotate("cube_pt.sdf", TEST.P3.C0, , , "MAXIMUM");
end
`endif /* __SDF__ */

endmodule
