#!/bin/sh
################################################################
# $Id: edgen.sh 123 2010-08-15 10:08:52Z matutani $
# Usage example:
#     ./edgen.sh port.loc > cube.ed
################################################################
LOC_FILE=$1
MODULE_NAME=cube
TEXT_LAYER_POWER=609
TEXT_LAYER_NS=605		# North and south
TEXT_LAYER_EW=606		# East and west

STRAP_NUM=27
STRAP_START=3.6
STRAP_OFFSET=14.4
VDD_VSS_DIFF=2.3

TMP_FILE=tmp.txt
rm -f ${TMP_FILE}

################################################################
# 1. Parse the pin name and its location
################################################################
cat ${LOC_FILE} | \
	tr -d '(),' | \
	awk 'NF==9 && ($2=="in" || $2=="out"){print $1 "\t" $8 "\t" $9}' > \
	${TMP_FILE}

################################################################
# 2. Detect the X and Y edge coordinates
################################################################
X_EDGE=`cat ${TMP_FILE} | sort -r -n -k2 | head -n1 | cut -f2`
Y_EDGE=`cat ${TMP_FILE} | sort -r -n -k3 | head -n1 | cut -f3`

################################################################
# 3. Print out ED file (signals)
################################################################
cat ${TMP_FILE} | \
awk -vX_EDGE=${X_EDGE} -vY_EDGE=${Y_EDGE} -vMODULE_NAME=${MODULE_NAME} \
	-vTEXT_LAYER_NS=${TEXT_LAYER_NS} -vTEXT_LAYER_EW=${TEXT_LAYER_EW} \
	'{if ($3==0 || $3==Y_EDGE) {layer=TEXT_LAYER_NS;} \
	  else if ($2==0 || $2==X_EDGE) {layer=TEXT_LAYER_EW;} \
	  else {printf("Error!n");exit;} \
	  printf("LAYOUT TEXT \"%s\" %s %s %d %s\n", $1, $2, $3, layer, MODULE_NAME);}'
rm -f ${TMP_FILE}

################################################################
# 4. Print out ED file (power)
################################################################
X0=${STRAP_START};
X1=`echo "scale=5; ${X0}+${VDD_VSS_DIFF}" | bc`
for NUM in `seq 1 ${STRAP_NUM}`; do
	echo "LAYOUT TEXT VDD ${X0} 0.0 ${TEXT_LAYER_POWER} ${MODULE_NAME}"
	echo "LAYOUT TEXT VSS ${X1} 0.0 ${TEXT_LAYER_POWER} ${MODULE_NAME}"
	X0=`echo "scale=5; ${X0}+${STRAP_OFFSET}" | bc`
	X1=`echo "scale=5; ${X1}+${STRAP_OFFSET}" | bc`
done

################################################################
