###################################################################
# $Id: compile_dc.tcl 26 2010-07-24 17:50:13Z matutani $
###################################################################

#
# Suppress warning messages
#
set suppress_errors { OPT-171 EQN-10 UID-401 CMD-041 DDB-72 DDB-74 SEL-005 }

#
# File names
#
set TOP_MODULE "sb4"
set SRC_FILE   { ../design/sb4.v}
set DB_FILE    "${TOP_MODULE}.db"
set NET_FILE   "${TOP_MODULE}.vnet"
set SDF_FILE   "${TOP_MODULE}.sdf"
set SDC_FILE   "${TOP_MODULE}.sdc"
set REP_FILE   "${TOP_MODULE}.rep"

###################################################################
#
# Library setup
#
###################################################################

#
# Library directory
#
set LIB_DIR "/home/vdec/lib/fujitsu65/cs202_sc_io_lib/db"
set search_path  [concat "." $LIB_DIR $search_path]
set link_path "*"

#
# Libraries
#
set std_max_libs { cs202sz_uc_aob_f_m40_13v.db cs202sz_uc_core_f_m40_13v.db cs202sz_uc_dscan_f_m40_13v.db cs202sz_uc_eco_f_m40_13v.db cs202sz_uc_iso_f_m40_13v.db cs202sz_uc_nscan_f_m40_13v.db cs202sz_uc_scan_f_m40_13v.db }
set std_typ_libs { cs202sz_uc_aob_t_p25_12v.db cs202sz_uc_core_t_p25_12v.db cs202sz_uc_dscan_t_p25_12v.db cs202sz_uc_eco_t_p25_12v.db cs202sz_uc_iso_t_p25_12v.db cs202sz_uc_nscan_t_p25_12v.db cs202sz_uc_scan_t_p25_12v.db }
set std_min_libs { cs202sz_uc_aob_s_p125_105v.db cs202sz_uc_core_s_p125_105v.db cs202sz_uc_dscan_s_p125_105v.db cs202sz_uc_eco_s_p125_105v.db cs202sz_uc_iso_s_p125_105v.db cs202sz_uc_nscan_s_p125_105v.db cs202sz_uc_scan_s_p125_105v.db }
# set mem_max_libs { cs202_cc_s_p100_11v.db }
# set mem_typ_libs { cs202_cc_t_p25_12v.db }
# set mem_min_libs { cs202_cc_f_m40_13v.db }
set mem_max_libs { }
set mem_typ_libs { }
set mem_min_libs { }
set iob_min_libs { cs202_io_f_m40_m_13v_36v.db }
set iob_typ_libs { cs202_io_t_p25_m_12v_33v.db }
set iob_max_libs { cs202_io_s_p125_m_105v_30v.db }
set LIB_MAX_FILE [concat $std_max_libs $mem_max_libs $iob_max_libs ]
set LIB_TYP_FILE [concat $std_typ_libs $mem_typ_libs $iob_typ_libs ]
set LIB_MIN_FILE [concat $std_min_libs $mem_min_libs $iob_min_libs ]
set DW_FILE      "dw_foundation.sldb"

#
# Read standard cell library
#
#set link_library   [concat $LIB_MAX_FILE $LIB_MIN_FILE $DW_FILE {*} ]
#set target_library [concat $LIB_MAX_FILE $LIB_MIN_FILE ]
#set symbol_library [concat $LIB_MAX_FILE $LIB_MIN_FILE "EXD.sdb" ]
#set link_library   [concat $LIB_TYP_FILE $DW_FILE {*} ]
#set target_library [concat $LIB_TYP_FILE ]
#set symbol_library [concat $LIB_TYP_FILE "EXD.sdb" ]
set link_library                 [concat $LIB_MAX_FILE $LIB_TYP_FILE $LIB_MIN_FILE $DW_FILE {*}]
set target_library               [concat $LIB_MAX_FILE $LIB_TYP_FILE $LIB_MIN_FILE             ]
set symbol_library               [concat $LIB_MAX_FILE $LIB_TYP_FILE $LIB_MIN_FILE "EXD.sdb"   ]

define_design_lib WORK -path ./WORK

###################################################################
#
# Design constraints
#
###################################################################

#
# Clock period [psec]
#
set clock_period	5000

#
# Clock edges
#
set clock_rise_edge     0
set clock_fall_edge     [expr $clock_period/2 + $clock_rise_edge]
set clock_wave          [list $clock_rise_edge $clock_fall_edge]

#
# Maxmin Clock Latency
#
set max_clock_latency   100
set min_clock_latency   0

#
# Clock Uncertainty
#
set clock_uncertainty   100

#
# Max Fanout
#
set max_fanout          10

#
# Conditions
#
set best_condition      "DEFAULT"
set worst_condition     "DEFAULT"

#
# Footprint Cell
#
set inv_footprint       "SC23INVXA1"
set nand_footprint      "SC23NAND2XA1"
set max_uc_core_lib     "cs202sz_uc_core_f_m40_13v"
set typ_uc_core_lib     "cs202sz_uc_core_t_p25_12v"
#set load_cell_port      "${max_uc_core_lib}/${nand_footprint}/YB"
set load_cell_port      "${typ_uc_core_lib}/${nand_footprint}/YB"

#
# Dont' Use Cells
#
set_dont_use      "cs202sz_uc_core_t_p25_12v/SC23NAND3XC1"
# set_dont_use      "cs202sz_uc_core_f_m40_13v/SC23NAND3XC1"
# set_dont_use		"cs202sz_uc_core_t_p25_12v/SC23NAND3XC1"

#
# Input/Output Delay
#
set input_delay		500
set output_delay	500

###################################################################
#
# Synthesize
#
###################################################################

#
# Read Verilog source files with clock gating
#
analyze -format verilog $SRC_FILE
elaborate $TOP_MODULE
current_design $TOP_MODULE
uniquify

#
# Clock gating (latch-based or latch-free)
#
# set_clock_gating_style -minimum_bitwidth 2 -sequential_cell latch
insert_clock_gating

#
# Create clocks 
#
create_clock -name clk -period $clock_period -w ${clock_wave} [find port clk]

set_input_delay $input_delay -clock clk [remove_from_collection [all_inputs] {clk ireset}]
set_driving_cell -lib_cell $inv_footprint -library $typ_uc_core_lib [remove_from_collection [all_inputs] {clk ireset}]

# set_fix_hold [get_clocks {clk}]
# set_clock_uncertainty  $clock_uncertainty [get_clocks {clk}]

set_ideal_network [find port clk]
set_ideal_network [find port ireset]

set_output_delay $output_delay -clock clk [all_outputs]
set_clock_latency -max $max_clock_latency [get_clocks {clk}]
set_clock_latency -min $min_clock_latency [get_clocks {clk}]

derive_clocks
set_load [expr 5 * [load_of $load_cell_port]] [all_outputs]
set_max_fanout $max_fanout [current_design]

#
# Compile!
#
# compile -ungroup_all -map_effort high
compile -map_effort high
compile -incremental_mapping -map_effort high
check_design

#
# Astro is case insensitive 
#
define_name_rules CAPS_ONLY -allow "A-Z 0-9 _"
change_names -rules CAPS_ONLY -hier

###################################################################
#
# Reports
#
###################################################################

#
# Output (db vnet sdc)
#
# write -format db -hierarchy -xg_force_db -output $DB_FILE
write -format verilog -hierarchy -output $NET_FILE
write_sdc $SDC_FILE

#
# Reports
#
redirect         $REP_FILE { report_area }
redirect -append $REP_FILE { report_power -hier }
redirect -append $REP_FILE { report_reference -hier }
redirect -append $REP_FILE { report_timing -path full -delay max -max_paths 3 -nworst 1 }
redirect -append $REP_FILE { report_lib $typ_uc_core_lib }

quit
